@@ -3,7 +3,7 @@
 namespace MediaWiki\Extension\AbuseFilter\Hooks;
 
 use MediaWiki\Extension\AbuseFilter\Variables\VariableHolder;
-use RecentChange;
+use MediaWiki\RecentChanges\RecentChange;
 
 interface AbuseFilterGenerateGenericVarsHook {
 	/**
@@ -16,7 +16,7 @@ interface AbuseFilterGenerateGenericVarsHook {
 	 *     this is the entry. Null if it's for the current action being filtered.
 	 * @return bool|void True or no return value to continue or false to abort
 	 */
-	  public function onAbuseFilter_generateGenericVars(
+	public function onAbuseFilter_generateGenericVars(
 		VariableHolder $vars,
 		?RecentChange $rc
 	);
