@@ -736,12 +736,11 @@ class SpecialAbuseLog extends AbuseFilterSpecialPage {
 
 		// Load data
 		$vars = $this->varBlobStore->loadVarDump( $row );
-		$varsArray = $this->varManager->dumpAllVars( $vars, true );
 
 		// Prevent users seeing logs which contain protected variables that the user cannot see.
 		if (
 			$filter->isProtected() &&
-			!$this->afPermissionManager->canViewProtectedVariables( $performer, array_keys( $varsArray ) )
+			!$this->afPermissionManager->canViewProtectedVariables( $performer, array_keys( $vars->getVars() ) )
 				->isGood()
 		) {
 			$out->addWikiMsg( 'abusefilter-examine-protected-vars-permission' );
@@ -766,6 +765,7 @@ class SpecialAbuseLog extends AbuseFilterSpecialPage {
 		// shouldn't see them.
 		$userAuthority = $this->getAuthority();
 		$protectedVariableValuesShown = [];
+		$varsArray = $this->varManager->dumpAllVars( $vars, $this->afPermissionManager->getProtectedVariables() );
 		foreach ( $this->afPermissionManager->getProtectedVariables() as $protectedVariable ) {
 			if ( isset( $varsArray[$protectedVariable] ) ) {
 				// Try each variable at a time, as the user may be able to see some but not all of the
