@@ -92,8 +92,9 @@ function onTargetChange( blockTarget ) {
 						);
 						message = mw.message(
 							'checkuser-tempaccount-specialblock-ips',
-							ips.length
-						).escaped().replace( '$2', () => mw.language.listToText( ips ) );
+							ips.length,
+							Object.assign( mw.language.listToText( ips ) )
+						).parse();
 						message = new OO.ui.HtmlSnippet( message );
 					} else {
 						message = mw.message(
