@@ -2,6 +2,7 @@
 
 namespace MediaWiki\Extension\Math\Tests\WikiTexVC\MMLmappings;
 
+use MediaWiki\Extension\Math\Tests\WikiTexVC\MathServiceContainerTrait;
 use MediaWiki\Extension\Math\WikiTexVC\MMLmappings\BaseParsing;
 use MediaWiki\Extension\Math\WikiTexVC\Nodes\DQ;
 use MediaWiki\Extension\Math\WikiTexVC\Nodes\Fun1;
@@ -15,12 +16,13 @@ use PHPUnit\Framework\TestCase;
  * @covers \MediaWiki\Extension\Math\WikiTexVC\MMLmappings\BaseParsing
  */
 class BaseParsingTest extends TestCase {
+	use MathServiceContainerTrait;
 
 	public function testAccent() {
 		$node = new Fun1(
 			'\\widetilde',
-				( new Literal( 'a' ) )
-			);
+			( new Literal( 'a' ) )
+		);
 		$result = BaseParsing::accent( $node, [], null, 'widetilde', '007E' );
 		$this->assertStringContainsString( '~', $result );
 		$this->assertStringContainsString( 'mover', $result );
@@ -215,4 +217,21 @@ f(x,y,z) & = & x + y + z
 		$result = BaseParsing::namedOp( $node, [], [], '\\gcd' );
 		$this->assertStringContainsString( '>gcd</mi>', $result );
 	}
+
+	public function testSpace() {
+		$node = new Literal( '\\ ' );
+		$result = BaseParsing::macro( $node, [], [], '\\ ', '\\text{ }' );
+		$this->assertStringContainsString( '<mtext>&#160;</mtext>', $result );
+	}
+
+	public function testIgnoreMisplacedLimit() {
+		$node = new Literal( '\\limits ' );
+		$result = BaseParsing::limits( $node, [], [], '\\limits' );
+		$this->assertSame( '', $result, 'Misplaced limits should be ignored' );
+	}
+
+	protected function setUp(): void {
+		parent::setUp();
+		$this->setUpMathServiceContainer();
+	}
 }
