@@ -10,17 +10,16 @@
 
 namespace MediaWiki\Extension\VisualEditor;
 
-use ApiBase;
-use ApiBlockInfoTrait;
-use ApiMain;
-use ApiResult;
-use Article;
-use Config;
-use ContentHandler;
-use DerivativeContext;
-use ExtensionRegistry;
-use IBufferingStatsdDataFactory;
+use MediaWiki\Api\ApiBase;
+use MediaWiki\Api\ApiBlockInfoTrait;
+use MediaWiki\Api\ApiMain;
+use MediaWiki\Api\ApiResult;
+use MediaWiki\Config\Config;
+use MediaWiki\Content\ContentHandler;
 use MediaWiki\Content\Transform\ContentTransformer;
+use MediaWiki\Content\WikitextContent;
+use MediaWiki\Context\DerivativeContext;
+use MediaWiki\Context\RequestContext;
 use MediaWiki\EditPage\EditPage;
 use MediaWiki\EditPage\IntroMessageBuilder;
 use MediaWiki\EditPage\PreloadedContentBuilder;
@@ -28,22 +27,25 @@ use MediaWiki\EditPage\TextboxBuilder;
 use MediaWiki\Language\RawMessage;
 use MediaWiki\Logger\LoggerFactory;
 use MediaWiki\MediaWikiServices;
+use MediaWiki\Page\Article;
 use MediaWiki\Page\PageReference;
 use MediaWiki\Page\WikiPageFactory;
+use MediaWiki\Permissions\PermissionManager;
+use MediaWiki\Registration\ExtensionRegistry;
 use MediaWiki\Request\DerivativeRequest;
 use MediaWiki\Revision\RevisionLookup;
 use MediaWiki\SpecialPage\SpecialPageFactory;
 use MediaWiki\Title\Title;
 use MediaWiki\User\Options\UserOptionsLookup;
 use MediaWiki\User\TempUser\TempUserCreator;
+use MediaWiki\User\User;
 use MediaWiki\User\UserFactory;
+use MediaWiki\User\UserIdentity;
 use MediaWiki\Watchlist\WatchlistManager;
 use MessageLocalizer;
-use RequestContext;
-use User;
 use Wikimedia\Assert\Assert;
 use Wikimedia\ParamValidator\ParamValidator;
-use WikitextContent;
+use Wikimedia\Stats\StatsFactory;
 
 class ApiVisualEditor extends ApiBase {
 	use ApiBlockInfoTrait;
@@ -70,7 +72,7 @@ class ApiVisualEditor extends ApiBase {
 		UserOptionsLookup $userOptionsLookup,
 		WatchlistManager $watchlistManager,
 		ContentTransformer $contentTransformer,
-		IBufferingStatsdDataFactory $statsdDataFactory,
+		StatsFactory $statsFactory,
 		WikiPageFactory $wikiPageFactory,
 		IntroMessageBuilder $introMessageBuilder,
 		PreloadedContentBuilder $preloadedContentBuilder,
@@ -79,7 +81,7 @@ class ApiVisualEditor extends ApiBase {
 	) {
 		parent::__construct( $main, $name );
 		$this->setLogger( LoggerFactory::getInstance( 'VisualEditor' ) );
-		$this->setStats( $statsdDataFactory );
+		$this->setStatsFactory( $statsFactory );
 		$this->revisionLookup = $revisionLookup;
 		$this->tempUserCreator = $tempUserCreator;
 		$this->userFactory = $userFactory;
@@ -104,9 +106,8 @@ class ApiVisualEditor extends ApiBase {
 
 	/**
 	 * @see EditPage::getUserForPermissions
-	 * @return User
 	 */
-	private function getUserForPermissions() {
+	private function getUserForPermissions(): User {
 		$user = $this->getUser();
 		if ( $this->tempUserCreator->shouldAutoCreate( $user, 'edit' ) ) {
 			return $this->userFactory->newUnsavedTempUser(
@@ -118,9 +119,8 @@ class ApiVisualEditor extends ApiBase {
 
 	/**
 	 * @see ApiParse::getUserForPreview
-	 * @return User
 	 */
-	private function getUserForPreview() {
+	private function getUserForPreview(): UserIdentity {
 		$user = $this->getUser();
 		if ( $this->tempUserCreator->shouldAutoCreate( $user, 'edit' ) ) {
 			return $this->userFactory->newUnsavedTempUser(
@@ -323,14 +323,14 @@ class ApiVisualEditor extends ApiBase {
 				$builder = new TextboxBuilder();
 				$protectedClasses = $builder->getTextboxProtectionCSSClasses( $title );
 
-				// Simplified EditPage::getEditPermissionErrors()
+				// Simplified EditPage::getEditPermissionStatus()
 				// TODO: Use API
 				// action=query&prop=info&intestactions=edit&intestactionsdetail=full&errorformat=html&errorsuselocal=1
-				$permErrors = $permissionManager->getPermissionErrors(
-					'edit', $this->getUserForPermissions(), $title, 'full' );
-				if ( $permErrors ) {
+				$status = $permissionManager->getPermissionStatus(
+					'edit', $this->getUserForPermissions(), $title, PermissionManager::RIGOR_FULL );
+				if ( !$status->isGood() ) {
 					// Show generic permission errors, including page protection, user blocks, etc.
-					$notice = $this->getOutput()->formatPermissionsErrorMessage( $permErrors, 'edit' );
+					$notice = $this->getOutput()->formatPermissionStatus( $status, 'edit' );
 					// That method returns wikitext (eww), hack to get it parsed:
 					$notice = ( new RawMessage( '$1', [ $notice ] ) )->page( $title )->parseAsBlock();
 					// Invent a message key 'permissions-error' to store in $notices
@@ -359,14 +359,14 @@ class ApiVisualEditor extends ApiBase {
 				}
 
 				// Will be false e.g. if user is blocked or page is protected
-				$canEdit = !$permErrors;
+				$canEdit = $status->isGood();
 
 				$blockinfo = null;
 				// Blocked user notice
 				if ( $permissionManager->isBlockedFrom( $user, $title, true ) ) {
 					$block = $user->getBlock();
 					if ( $block ) {
-						// Already added to $notices via #getPermissionErrors above.
+						// Already added to $notices via #getPermissionStatus above.
 						// Add block info for MobileFrontend:
 						$blockinfo = $this->getBlockDetails( $block );
 					}
@@ -435,20 +435,29 @@ class ApiVisualEditor extends ApiBase {
 						&& $this->tempUserCreator->isAutoCreateAction( 'edit' )
 						&& $permissionManager->userHasRight( $user, 'createaccount' );
 
+				// phpcs:disable MediaWiki.WhiteSpace.SpaceBeforeSingleLineComment.NewLineComment
+				/** @phpcs-require-sorted-array */
 				$result = [
-					'result' => 'success',
-					'notices' => $notices,
-					'copyrightWarning' => $copyrightWarning,
+					// --------------------------------------------------------------------------------
+					// This should match ArticleTarget#getWikitextDataPromiseForDoc and ArticleTarget#storeDocState
+					// --------------------------------------------------------------------------------
+					'basetimestamp' => $baseTimestamp,
+					'blockinfo' => $blockinfo, // only used by MobileFrontend EditorGateway
+					'canEdit' => $canEdit,
 					'checkboxesDef' => $checkboxesDef,
 					'checkboxesMessages' => $checkboxesMessages,
-					'protectedClasses' => implode( ' ', $protectedClasses ),
-					'basetimestamp' => $baseTimestamp,
-					'starttimestamp' => wfTimestampNow(),
+					// 'content' => ..., // optional, see below
+					'copyrightWarning' => $copyrightWarning,
+					// 'etag' => ..., // optional, see below
+					'notices' => $notices,
 					'oldid' => $oldid,
-					'blockinfo' => $blockinfo,
+					// 'preloaded' => ..., // optional, see below
+					'protectedClasses' => implode( ' ', $protectedClasses ),
+					'result' => 'success', // probably unused?
+					'starttimestamp' => wfTimestampNow(),
 					'wouldautocreate' => $wouldautocreate,
-					'canEdit' => $canEdit,
 				];
+				// phpcs:enable MediaWiki.WhiteSpace.SpaceBeforeSingleLineComment.NewLineComment
 				if ( isset( $restbaseHeaders['etag'] ) ) {
 					$result['etag'] = $restbaseHeaders['etag'];
 				}
@@ -498,9 +507,40 @@ class ApiVisualEditor extends ApiBase {
 				break;
 		}
 
+		if (
+			is_array( $result ) &&
+			isset( $result['content'] ) &&
+			is_string( $result['content'] )
+		) {
+			// Protect content from being corrupted by conversion to Unicode NFC.
+			// Without this, MediaWiki::Api::ApiResult::addValue can break html tags.
+			// See T382756
+			$result['content'] = $this->makeSafeHtmlForNfc( $result['content'] );
+		}
+
 		$this->getResult()->addValue( null, $this->getModuleName(), $result );
 	}
 
+	/**
+	 * Protect html-like content from being corrupted by conversion to Unicode NFC.
+	 *
+	 * Encodes U+0338 COMBINING LONG SOLIDUS OVERLAY as an html numeric character reference.
+	 * Otherwise, conversion to Unicode NFC can break html tags by converting
+	 * '>' + U+0338 to U+226F (NOT GREATER THAN), and
+	 * '<' + U+0338 to U+226E (NOT LESS THAN)
+	 *
+	 * Note we cannot just search for those two combinations, because sequences of combining
+	 * characters can get reordered, e.g. '>' + U+0339 + U+0338 will become U+226F + U+0339.
+	 * See https://unicode.org/reports/tr15/
+	 *
+	 * @param string $html
+	 * @return string
+	 */
+	public static function makeSafeHtmlForNfc( string $html ) {
+		$html = str_replace( "\u{0338}", '&#x338;', $html );
+		return $html;
+	}
+
 	/**
 	 * Check if the configured allowed namespaces include the specified namespace
 	 *
@@ -508,7 +548,7 @@ class ApiVisualEditor extends ApiBase {
 	 * @param int $namespaceId Namespace ID
 	 * @return bool
 	 */
-	public static function isAllowedNamespace( Config $config, $namespaceId ) {
+	public static function isAllowedNamespace( Config $config, int $namespaceId ): bool {
 		return in_array( $namespaceId, self::getAvailableNamespaceIds( $config ), true );
 	}
 
@@ -518,7 +558,7 @@ class ApiVisualEditor extends ApiBase {
 	 * @param Config $config
 	 * @return int[]
 	 */
-	public static function getAvailableNamespaceIds( Config $config ) {
+	public static function getAvailableNamespaceIds( Config $config ): array {
 		$namespaceInfo = MediaWikiServices::getInstance()->getNamespaceInfo();
 		$configuredNamespaces = array_replace(
 			ExtensionRegistry::getInstance()->getAttribute( 'VisualEditorAvailableNamespaces' ),
@@ -541,7 +581,7 @@ class ApiVisualEditor extends ApiBase {
 	 * @param string $contentModel Content model ID
 	 * @return bool
 	 */
-	public static function isAllowedContentType( Config $config, $contentModel ) {
+	public static function isAllowedContentType( Config $config, string $contentModel ): bool {
 		$availableContentModels = array_merge(
 			ExtensionRegistry::getInstance()->getAttribute( 'VisualEditorAvailableContentModels' ),
 			$config->get( 'VisualEditorAvailableContentModels' )
