@@ -6,8 +6,6 @@ abstract class ConfigBuilderBase implements ConfigBuilder {
 
 	abstract protected function has( string $key ): bool;
 
-	abstract protected function get( string $key );
-
 	abstract protected function update( string $key, $value );
 
 	/**
@@ -16,9 +14,9 @@ abstract class ConfigBuilderBase implements ConfigBuilder {
 	public function set(
 		string $key,
 		$newValue,
-		MergeStrategy $mergeStrategy = null
+		?MergeStrategy $mergeStrategy = null
 	): ConfigBuilder {
-		if ( $mergeStrategy && is_array( $newValue ) ) {
+		if ( $mergeStrategy && $this->has( $key ) && is_array( $newValue ) ) {
 			$oldValue = $this->get( $key );
 			if ( $oldValue && is_array( $oldValue ) ) {
 				$newValue = $mergeStrategy->merge( $oldValue, $newValue );
@@ -28,10 +26,23 @@ abstract class ConfigBuilderBase implements ConfigBuilder {
 		return $this;
 	}
 
+	/**
+	 * @inheritDoc
+	 */
+	public function setMulti( array $values, array $mergeStrategies = [] ): ConfigBuilder {
+		foreach ( $values as $key => $value ) {
+			$this->set( $key, $value, $mergeStrategies[$key] ?? null );
+		}
+		return $this;
+	}
+
+	/**
+	 * @inheritDoc
+	 */
 	public function setDefault(
 		string $key,
 		$defaultValue,
-		MergeStrategy $mergeStrategy = null
+		?MergeStrategy $mergeStrategy = null
 	): ConfigBuilder {
 		if ( $this->has( $key ) ) {
 			if ( $mergeStrategy && $defaultValue && is_array( $defaultValue ) ) {
@@ -48,4 +59,14 @@ abstract class ConfigBuilderBase implements ConfigBuilder {
 		return $this;
 	}
 
+	/**
+	 * @inheritDoc
+	 */
+	public function setMultiDefault( array $defaults, array $mergeStrategies ): ConfigBuilder {
+		foreach ( $defaults as $key => $defaultValue ) {
+			$this->setDefault( $key, $defaultValue, $mergeStrategies[$key] ?? null );
+		}
+		return $this;
+	}
+
 }
