@@ -2,11 +2,11 @@
 
 namespace MediaWiki\Storage\Hook;
 
-use Content;
 use MediaWiki\CommentStore\CommentStoreComment;
+use MediaWiki\Content\Content;
+use MediaWiki\Page\WikiPage;
+use MediaWiki\User\User;
 use StatusValue;
-use User;
-use WikiPage;
 
 /**
  * This is a hook handler interface, see docs/Hooks.md.
