@@ -20,17 +20,27 @@
  * @file
  */
 
+namespace MediaWiki\Api;
+
+use MediaWiki\ChangeTags\ChangeTagsStore;
 use MediaWiki\CommentFormatter\CommentFormatter;
 use MediaWiki\CommentFormatter\RowCommentFormatter;
 use MediaWiki\CommentStore\CommentStore;
+use MediaWiki\Logging\DatabaseLogEntry;
+use MediaWiki\Logging\LogEventsList;
+use MediaWiki\Logging\LogFormatterFactory;
+use MediaWiki\Logging\LogPage;
 use MediaWiki\MainConfigNames;
 use MediaWiki\ParamValidator\TypeDef\NamespaceDef;
 use MediaWiki\ParamValidator\TypeDef\UserDef;
 use MediaWiki\Storage\NameTableAccessException;
 use MediaWiki\Storage\NameTableStore;
 use MediaWiki\Title\Title;
+use MediaWiki\User\UserNameUtils;
 use Wikimedia\ParamValidator\ParamValidator;
 use Wikimedia\ParamValidator\TypeDef\IntegerDef;
+use Wikimedia\Rdbms\IExpression;
+use Wikimedia\Rdbms\LikeValue;
 
 /**
  * Query action to List the log events, with optional filtering by various parameters.
@@ -39,42 +49,45 @@ use Wikimedia\ParamValidator\TypeDef\IntegerDef;
  */
 class ApiQueryLogEvents extends ApiQueryBase {
 
-	/** @var CommentStore */
-	private $commentStore;
-
-	/** @var CommentFormatter */
-	private $commentFormatter;
-
-	/** @var NameTableStore */
-	private $changeTagDefStore;
+	private CommentStore $commentStore;
+	private CommentFormatter $commentFormatter;
+	private NameTableStore $changeTagDefStore;
+	private ChangeTagsStore $changeTagsStore;
+	private UserNameUtils $userNameUtils;
+	private LogFormatterFactory $logFormatterFactory;
 
 	/** @var string[]|null */
 	private $formattedComments;
 
-	/**
-	 * @param ApiQuery $query
-	 * @param string $moduleName
-	 * @param CommentStore $commentStore
-	 * @param RowCommentFormatter $commentFormatter
-	 * @param NameTableStore $changeTagDefStore
-	 */
 	public function __construct(
 		ApiQuery $query,
-		$moduleName,
+		string $moduleName,
 		CommentStore $commentStore,
 		RowCommentFormatter $commentFormatter,
-		NameTableStore $changeTagDefStore
+		NameTableStore $changeTagDefStore,
+		ChangeTagsStore $changeTagsStore,
+		UserNameUtils $userNameUtils,
+		LogFormatterFactory $logFormatterFactory
 	) {
 		parent::__construct( $query, $moduleName, 'le' );
 		$this->commentStore = $commentStore;
 		$this->commentFormatter = $commentFormatter;
 		$this->changeTagDefStore = $changeTagDefStore;
+		$this->changeTagsStore = $changeTagsStore;
+		$this->userNameUtils = $userNameUtils;
+		$this->logFormatterFactory = $logFormatterFactory;
 	}
 
-	private $fld_ids = false, $fld_title = false, $fld_type = false,
-		$fld_user = false, $fld_userid = false,
-		$fld_timestamp = false, $fld_comment = false, $fld_parsedcomment = false,
-		$fld_details = false, $fld_tags = false;
+	private bool $fld_ids = false;
+	private bool $fld_title = false;
+	private bool $fld_type = false;
+	private bool $fld_user = false;
+	private bool $fld_userid = false;
+	private bool $fld_timestamp = false;
+	private bool $fld_comment = false;
+	private bool $fld_parsedcomment = false;
+	private bool $fld_details = false;
+	private bool $fld_tags = false;
 
 	public function execute() {
 		$params = $this->extractRequestParams();
@@ -99,10 +112,7 @@ class ApiQueryLogEvents extends ApiQueryBase {
 			$this->addWhere( $hideLogs );
 		}
 
-		$this->addTables( [ 'logging', 'actor' ] );
-		$this->addJoinConds( [
-			'actor' => [ 'JOIN', 'actor_id=log_actor' ],
-		] );
+		$this->addTables( 'logging' );
 
 		$this->addFields( [
 			'log_id',
@@ -112,6 +122,19 @@ class ApiQueryLogEvents extends ApiQueryBase {
 			'log_deleted',
 		] );
 
+		$user = $params['user'];
+		if ( $this->fld_user || $this->fld_userid || $user !== null ) {
+			$this->addTables( 'actor' );
+			$this->addJoinConds( [
+				'actor' => [ 'JOIN', 'actor_id=log_actor' ],
+			] );
+			$this->addFieldsIf( [ 'actor_name', 'actor_user' ], $this->fld_user );
+			$this->addFieldsIf( 'actor_user', $this->fld_userid );
+			if ( $user !== null ) {
+				$this->addWhereFld( 'actor_name', $user );
+			}
+		}
+
 		if ( $this->fld_ids ) {
 			$this->addTables( 'page' );
 			$this->addJoinConds( [
@@ -124,8 +147,6 @@ class ApiQueryLogEvents extends ApiQueryBase {
 			// scenarios, e.g. deletion, recreation.
 			$this->addFields( [ 'page_id', 'log_page' ] );
 		}
-		$this->addFieldsIf( [ 'actor_name', 'actor_user' ], $this->fld_user );
-		$this->addFieldsIf( 'actor_user', $this->fld_userid );
 		$this->addFieldsIf(
 			[ 'log_namespace', 'log_title' ],
 			$this->fld_title || $this->fld_parsedcomment
@@ -140,7 +161,9 @@ class ApiQueryLogEvents extends ApiQueryBase {
 		}
 
 		if ( $this->fld_tags ) {
-			$this->addFields( [ 'ts_tags' => ChangeTags::makeTagSummarySubquery( 'logging' ) ] );
+			$this->addFields( [
+				'ts_tags' => $this->changeTagsStore->makeTagSummarySubquery( 'logging' )
+			] );
 		}
 
 		if ( $params['tag'] !== null ) {
@@ -205,11 +228,6 @@ class ApiQueryLogEvents extends ApiQueryBase {
 		$limit = $params['limit'];
 		$this->addOption( 'LIMIT', $limit + 1 );
 
-		$user = $params['user'];
-		if ( $user !== null ) {
-			$this->addWhereFld( 'actor_name', $user );
-		}
-
 		$title = $params['title'];
 		if ( $title !== null ) {
 			$titleObj = Title::newFromText( $title );
@@ -236,7 +254,9 @@ class ApiQueryLogEvents extends ApiQueryBase {
 				$this->dieWithError( [ 'apierror-invalidtitle', wfEscapeWikiText( $prefix ) ] );
 			}
 			$this->addWhereFld( 'log_namespace', $title->getNamespace() );
-			$this->addWhere( 'log_title ' . $db->buildLike( $title->getDBkey(), $db->anyString() ) );
+			$this->addWhere(
+				$db->expr( 'log_title', IExpression::LIKE, new LikeValue( $title->getDBkey(), $db->anyString() ) )
+			);
 		}
 
 		// Paranoia: avoid brute force searches (T19342)
@@ -308,7 +328,7 @@ class ApiQueryLogEvents extends ApiQueryBase {
 		$result->addIndexedTagName( [ 'query', $this->getModuleName() ], 'item' );
 	}
 
-	private function extractRowInfo( $row ) {
+	private function extractRowInfo( \stdClass $row ): array {
 		$logEntry = DatabaseLogEntry::newFromRow( $row );
 		$vals = [
 			ApiResult::META_TYPE => 'assoc',
@@ -324,7 +344,7 @@ class ApiQueryLogEvents extends ApiQueryBase {
 		}
 
 		$authority = $this->getAuthority();
-		if ( $this->fld_title || $this->fld_ids || $this->fld_details && $row->log_params !== '' ) {
+		if ( $this->fld_title || $this->fld_ids || ( $this->fld_details && $row->log_params !== '' ) ) {
 			if ( LogEventsList::isDeleted( $row, LogPage::DELETED_ACTION ) ) {
 				$vals['actionhidden'] = true;
 				$anyHidden = true;
@@ -344,7 +364,7 @@ class ApiQueryLogEvents extends ApiQueryBase {
 					}
 				}
 				if ( $this->fld_details ) {
-					$vals['params'] = LogFormatter::newFromEntry( $logEntry )->formatParametersForApi();
+					$vals['params'] = $this->logFormatterFactory->newFromEntry( $logEntry )->formatParametersForApi();
 				}
 			}
 		}
@@ -367,6 +387,10 @@ class ApiQueryLogEvents extends ApiQueryBase {
 					$vals['userid'] = (int)$row->actor_user;
 				}
 
+				if ( isset( $vals['user'] ) && $this->userNameUtils->isTemp( $vals['user'] ) ) {
+					$vals['temp'] = true;
+				}
+
 				if ( !$row->actor_user ) {
 					$vals['anon'] = true;
 				}
@@ -483,10 +507,14 @@ class ApiQueryLogEvents extends ApiQueryBase {
 					'older'
 				],
 				ApiBase::PARAM_HELP_MSG => 'api-help-param-direction',
+				ApiBase::PARAM_HELP_MSG_PER_VALUE => [
+					'newer' => 'api-help-paramvalue-direction-newer',
+					'older' => 'api-help-paramvalue-direction-older',
+				],
 			],
 			'user' => [
 				ParamValidator::PARAM_TYPE => 'user',
-				UserDef::PARAM_ALLOWED_USER_TYPES => [ 'name', 'ip', 'id', 'interwiki' ],
+				UserDef::PARAM_ALLOWED_USER_TYPES => [ 'name', 'ip', 'temp', 'id', 'interwiki' ],
 			],
 			'title' => null,
 			'namespace' => [
@@ -525,3 +553,6 @@ class ApiQueryLogEvents extends ApiQueryBase {
 		return 'https://www.mediawiki.org/wiki/Special:MyLanguage/API:Logevents';
 	}
 }
+
+/** @deprecated class alias since 1.43 */
+class_alias( ApiQueryLogEvents::class, 'ApiQueryLogEvents' );
