@@ -2,8 +2,8 @@
 
 namespace MediaWiki\Api\Hook;
 
-use ApiOptions;
-use User;
+use MediaWiki\Api\ApiOptions;
+use MediaWiki\User\User;
 
 /**
  * This is a hook handler interface, see docs/Hooks.md.
@@ -23,7 +23,7 @@ interface ApiOptionsHook {
 	 * @param User $user User object whose preferences are being changed
 	 * @param array $changes Associative array of preference name => value
 	 * @param string[] $resetKinds Array of strings specifying which options kinds to reset
-	 *   See User::resetOptions() and User::getOptionKinds() for possible values.
+	 *   See PreferencesFactory::listResetKinds() for possible values.
 	 * @return bool|void True or no return value to continue or false to abort
 	 */
 	public function onApiOptions( $apiModule, $user, $changes, $resetKinds );
