@@ -16,19 +16,30 @@
  * http://www.gnu.org/copyleft/gpl.html
  *
  * @file
- * @ingroup FileRepo
  */
 
+namespace MediaWiki\FileRepo;
+
+use LogicException;
+use MediaTransformError;
+use MediaWiki\FileRepo\File\File;
+use MediaWiki\FileRepo\File\ForeignAPIFile;
+use MediaWiki\Json\FormatJson;
 use MediaWiki\Linker\LinkTarget;
 use MediaWiki\Logger\LoggerFactory;
+use MediaWiki\MainConfigNames;
 use MediaWiki\MediaWikiServices;
 use MediaWiki\Page\PageIdentity;
+use MediaWiki\Title\Title;
+use RuntimeException;
+use Wikimedia\FileBackend\FileBackend;
+use Wikimedia\ObjectCache\WANObjectCache;
 
 /**
  * A foreign repository for a remote MediaWiki accessible through api.php requests.
  *
- * Example config:
- *
+ * @par Example config:
+ * @code
  * $wgForeignFileRepos[] = [
  *   'class'                  => ForeignAPIRepo::class,
  *   'name'                   => 'shared',
@@ -36,6 +47,7 @@ use MediaWiki\Page\PageIdentity;
  *   'fetchDescription'       => true, // Optional
  *   'descriptionCacheExpiry' => 3600,
  * ];
+ * @endcode
  *
  * @ingroup FileRepo
  */
@@ -53,25 +65,26 @@ class ForeignAPIRepo extends FileRepo implements IForeignRepoWithMWApi {
 		'timestamp',
 	];
 
+	/** @var callable */
 	protected $fileFactory = [ ForeignAPIFile::class, 'newFromTitle' ];
 	/** @var int Check back with Commons after this expiry */
-	protected $apiThumbCacheExpiry = 86400; // 1 day (24*3600)
+	protected $apiThumbCacheExpiry = 24 * 3600; // 1 day
 
 	/** @var int Redownload thumbnail files after this expiry */
-	protected $fileCacheExpiry = 2592000; // 1 month (30*24*3600)
+	protected $fileCacheExpiry = 30 * 24 * 3600; // 1 month
 
 	/**
 	 * @var int API metadata cache time.
 	 * @since 1.38
 	 *
 	 * This is often the performance bottleneck for ForeignAPIRepo. For
-	 * each file used, we must fetch file metadata for it and every high-dpi
+	 * each file used, we must fetch file metadata for it and every high-DPI
 	 * variant, in serial, during the parse. This is slow if a page has many
 	 * files, with RTT of the handshake often being significant. The metadata
 	 * rarely changes, but if a new version of the file was uploaded, it might
 	 * be displayed incorrectly until its metadata entry falls out of cache.
 	 */
-	protected $apiMetadataExpiry = 14400; // 4 hours
+	protected $apiMetadataExpiry = 4 * 3600; // 4 hours
 
 	/** @var array */
 	protected $mFileExists = [];
@@ -83,7 +96,8 @@ class ForeignAPIRepo extends FileRepo implements IForeignRepoWithMWApi {
 	 * @param array|null $info
 	 */
 	public function __construct( $info ) {
-		$localFileRepo = MediaWikiServices::getInstance()->getMainConfig()->get( 'LocalFileRepo' );
+		$localFileRepo = MediaWikiServices::getInstance()->getMainConfig()
+			->get( MainConfigNames::LocalFileRepo );
 		parent::__construct( $info );
 
 		// https://commons.wikimedia.org/w/api.php
@@ -111,19 +125,12 @@ class ForeignAPIRepo extends FileRepo implements IForeignRepoWithMWApi {
 		}
 	}
 
-	/**
-	 * @return string
-	 */
-	private function getApiUrl() {
-		return $this->mApiBase;
-	}
-
 	/**
 	 * Per docs in FileRepo, this needs to return false if we don't support versioned
 	 * files. Well, we don't.
 	 *
 	 * @param PageIdentity|LinkTarget|string $title
-	 * @param string|bool $time
+	 * @param string|false $time
 	 * @return File|false
 	 */
 	public function newFile( $title, $time = false ) {
@@ -204,7 +211,8 @@ class ForeignAPIRepo extends FileRepo implements IForeignRepoWithMWApi {
 	 * @return array|null
 	 */
 	public function fetchImageQuery( $query ) {
-		$languageCode = MediaWikiServices::getInstance()->getMainConfig()->get( 'LanguageCode' );
+		$languageCode = MediaWikiServices::getInstance()->getMainConfig()
+			->get( MainConfigNames::LanguageCode );
 
 		$query = array_merge( $query,
 			[
@@ -228,7 +236,7 @@ class ForeignAPIRepo extends FileRepo implements IForeignRepoWithMWApi {
 
 	/**
 	 * @param array $data
-	 * @return bool|array
+	 * @return array|false
 	 */
 	public function getImageInfo( $data ) {
 		if ( $data && isset( $data['query']['pages'] ) ) {
@@ -307,7 +315,7 @@ class ForeignAPIRepo extends FileRepo implements IForeignRepoWithMWApi {
 	 * @param int $height
 	 * @param string $otherParams
 	 * @param string|null $lang Language code for language of error
-	 * @return bool|MediaTransformError
+	 * @return MediaTransformError|false
 	 * @since 1.22
 	 */
 	public function getThumbError(
@@ -350,7 +358,7 @@ class ForeignAPIRepo extends FileRepo implements IForeignRepoWithMWApi {
 	 * @param int $height
 	 * @param string $params Other rendering parameters (page number, etc)
 	 *   from handler's makeParamString.
-	 * @return bool|string
+	 * @return string|false
 	 */
 	public function getThumbUrlFromCache( $name, $width, $height, $params = "" ) {
 		// We can't check the local cache using FileRepo functions because
@@ -463,7 +471,7 @@ class ForeignAPIRepo extends FileRepo implements IForeignRepoWithMWApi {
 	/**
 	 * Get the local directory corresponding to one of the basic zones
 	 * @param string $zone
-	 * @return bool|null|string
+	 * @return null|string|false
 	 */
 	public function getZonePath( $zone ) {
 		$supported = [ 'public', 'thumb' ];
@@ -487,8 +495,10 @@ class ForeignAPIRepo extends FileRepo implements IForeignRepoWithMWApi {
 	 * @return string
 	 */
 	public static function getUserAgent() {
-		return MediaWikiServices::getInstance()->getHttpRequestFactory()->getUserAgent() .
-			" ForeignAPIRepo/" . self::VERSION;
+		$mediaWikiVersion = MediaWikiServices::getInstance()->getHttpRequestFactory()->getUserAgent();
+		$classVersion = self::VERSION;
+		$contactUrl = MediaWikiServices::getInstance()->getUrlUtils()->getCanonicalServer();
+		return "$mediaWikiVersion ($contactUrl) ForeignAPIRepo/$classVersion";
 	}
 
 	/**
@@ -499,7 +509,7 @@ class ForeignAPIRepo extends FileRepo implements IForeignRepoWithMWApi {
 	 */
 	public function getInfo() {
 		$info = parent::getInfo();
-		$info['apiurl'] = $this->getApiUrl();
+		$info['apiurl'] = $this->mApiBase;
 
 		$query = [
 			'format' => 'json',
@@ -534,10 +544,15 @@ class ForeignAPIRepo extends FileRepo implements IForeignRepoWithMWApi {
 	public static function httpGet(
 		$url, $timeout = 'default', $options = [], &$mtime = false
 	) {
+		$urlUtils = MediaWikiServices::getInstance()->getUrlUtils();
+		$requestFactory = MediaWikiServices::getInstance()->getHttpRequestFactory();
+
 		$options['timeout'] = $timeout;
-		/* Http::get */
-		$url = wfExpandUrl( $url, PROTO_HTTP );
+		$url = $urlUtils->expand( $url, PROTO_HTTP );
 		wfDebug( "ForeignAPIRepo: HTTP GET: $url" );
+		if ( !$url ) {
+			return false;
+		}
 		$options['method'] = "GET";
 
 		if ( !isset( $options['timeout'] ) ) {
@@ -546,8 +561,8 @@ class ForeignAPIRepo extends FileRepo implements IForeignRepoWithMWApi {
 
 		$options['userAgent'] = self::getUserAgent();
 
-		$req = MediaWikiServices::getInstance()->getHttpRequestFactory()
-			->create( $url, $options, __METHOD__ );
+		$req = $requestFactory->create( $url, $options, __METHOD__ );
+		$req->setHeader( 'Referer', $urlUtils->getCanonicalServer() );
 		$status = $req->execute();
 
 		if ( $status->isOK() ) {
@@ -589,7 +604,10 @@ class ForeignAPIRepo extends FileRepo implements IForeignRepoWithMWApi {
 		}
 
 		return $this->wanCache->getWithSetCallback(
-			$this->getLocalCacheKey( $attribute, sha1( $url ) ),
+			// Allow reusing the same cached data across wikis (T285271).
+			// This does not use getSharedCacheKey() because caching here
+			// is transparent to client wikis (which are not expected to issue purges).
+			$this->wanCache->makeGlobalKey( "filerepo-$attribute", sha1( $url ) ),
 			$cacheTTL,
 			function ( $curValue, &$ttl ) use ( $url ) {
 				$html = self::httpGet( $url, 'default', [], $mtime );
@@ -610,17 +628,19 @@ class ForeignAPIRepo extends FileRepo implements IForeignRepoWithMWApi {
 
 	/**
 	 * @param callable $callback
-	 * @throws MWException
+	 * @return never
 	 */
 	public function enumFiles( $callback ) {
-		// @phan-suppress-previous-line PhanPluginNeverReturnMethod
-		throw new MWException( 'enumFiles is not supported by ' . static::class );
+		throw new RuntimeException( 'enumFiles is not supported by ' . static::class );
 	}
 
 	/**
-	 * @throws MWException
+	 * @return never
 	 */
 	protected function assertWritableRepo() {
-		throw new MWException( static::class . ': write operations are not supported.' );
+		throw new LogicException( static::class . ': write operations are not supported.' );
 	}
 }
+
+/** @deprecated class alias since 1.44 */
+class_alias( ForeignAPIRepo::class, 'ForeignAPIRepo' );
