@@ -20,7 +20,16 @@
  * @file
  */
 
+use MediaWiki\Context\ContextSource;
+use MediaWiki\Context\IContextSource;
+use MediaWiki\Context\RequestContext;
+use MediaWiki\FileRepo\File\File;
+use MediaWiki\HookContainer\HookRunner;
+use MediaWiki\Language\Language;
+use MediaWiki\MainConfigNames;
 use MediaWiki\MediaWikiServices;
+use MediaWiki\Parser\Parser;
+use MediaWiki\Title\Title;
 
 /**
  * Image gallery
@@ -35,7 +44,7 @@ abstract class ImageGalleryBase extends ContextSource {
 
 	/**
 	 * @var array[] Gallery images
-	 * @phan-var array<int,array{0:Title,1:string,2:string,3:string,4:array,5:int}>
+	 * @phan-var array<int,array{0:Title,1:string,2:?string,3:string,4:array,5:int,6:?array}>
 	 */
 	protected $mImages;
 
@@ -113,13 +122,13 @@ abstract class ImageGalleryBase extends ContextSource {
 	 * @return ImageGalleryBase
 	 * @throws ImageGalleryClassNotFoundException
 	 */
-	public static function factory( $mode = false, IContextSource $context = null ) {
+	public static function factory( $mode = false, ?IContextSource $context = null ) {
 		self::loadModes();
 		if ( !$context ) {
 			$context = RequestContext::getMainAndWarn( __METHOD__ );
 		}
 		if ( !$mode ) {
-			$galleryOptions = $context->getConfig()->get( 'GalleryOptions' );
+			$galleryOptions = $context->getConfig()->get( MainConfigNames::GalleryOptions );
 			$mode = $galleryOptions['mode'];
 		}
 
@@ -144,7 +153,8 @@ abstract class ImageGalleryBase extends ContextSource {
 				'slideshow' => SlideshowImageGallery::class,
 			];
 			// Allow extensions to make a new gallery format.
-			Hooks::runner()->onGalleryGetModes( self::$modeMapping );
+			( new HookRunner( MediaWikiServices::getInstance()->getHookContainer() ) )
+				->onGalleryGetModes( self::$modeMapping );
 		}
 	}
 
@@ -161,12 +171,12 @@ abstract class ImageGalleryBase extends ContextSource {
 	 * @param string $mode
 	 * @param IContextSource|null $context
 	 */
-	public function __construct( $mode = 'traditional', IContextSource $context = null ) {
+	public function __construct( $mode = 'traditional', ?IContextSource $context = null ) {
 		if ( $context ) {
 			$this->setContext( $context );
 		}
 
-		$galleryOptions = $this->getConfig()->get( 'GalleryOptions' );
+		$galleryOptions = $this->getConfig()->get( MainConfigNames::GalleryOptions );
 		$this->mImages = [];
 		$this->mShowBytes = $galleryOptions['showBytes'];
 		$this->mShowDimensions = $galleryOptions['showDimensions'];
@@ -238,7 +248,12 @@ abstract class ImageGalleryBase extends ContextSource {
 	 *   and those below 0 are ignored.
 	 */
 	public function setWidths( $num ) {
-		$parsed = Parser::parseWidthParam( $num, false );
+		$parser = $this->mParser;
+		if ( !$parser ) {
+			wfDeprecated( __METHOD__ . ' without parser', '1.43' );
+			$parser = MediaWikiServices::getInstance()->getParser();
+		}
+		$parsed = $parser->parseWidthParam( $num, false );
 		if ( isset( $parsed['width'] ) && $parsed['width'] > 0 ) {
 			$this->mWidths = $parsed['width'];
 		}
@@ -251,7 +266,12 @@ abstract class ImageGalleryBase extends ContextSource {
 	 *   and those below 0 are ignored.
 	 */
 	public function setHeights( $num ) {
-		$parsed = Parser::parseWidthParam( $num, false );
+		$parser = $this->mParser;
+		if ( !$parser ) {
+			wfDeprecated( __METHOD__ . ' without parser', '1.43' );
+			$parser = MediaWikiServices::getInstance()->getParser();
+		}
+		$parsed = $parser->parseWidthParam( $num, false );
 		if ( isset( $parsed['width'] ) && $parsed['width'] > 0 ) {
 			$this->mHeights = $parsed['width'];
 		}
@@ -275,10 +295,11 @@ abstract class ImageGalleryBase extends ContextSource {
 	 * @param Title $title Title object of the image that is added to the gallery
 	 * @param string $html Additional HTML text to be shown. The name and size
 	 *   of the image are always shown.
-	 * @param string $alt Alt text for the image
+	 * @param string|null $alt Alt text for the image, or null to omit
 	 * @param string $link Override image link (optional)
 	 * @param array $handlerOpts Array of options for image handler (aka page number)
 	 * @param int $loading Sets loading attribute of the underlying <img> (optional)
+	 * @param ?array $imageOptions To supercede the $link param
 	 */
 	public function add(
 			$title,
@@ -286,13 +307,14 @@ abstract class ImageGalleryBase extends ContextSource {
 			$alt = '',
 			$link = '',
 			$handlerOpts = [],
-			$loading = self::LOADING_DEFAULT
+			$loading = self::LOADING_DEFAULT,
+			?array $imageOptions = null
 		) {
 		if ( $title instanceof File ) {
 			// Old calling convention
 			$title = $title->getTitle();
 		}
-		$this->mImages[] = [ $title, $html, $alt, $link, $handlerOpts, $loading ];
+		$this->mImages[] = [ $title, $html, $alt, $link, $handlerOpts, $loading, $imageOptions ];
 		wfDebug( 'ImageGallery::add ' . $title->getText() );
 	}
 
@@ -302,10 +324,11 @@ abstract class ImageGalleryBase extends ContextSource {
 	 * @param Title $title Title object of the image that is added to the gallery
 	 * @param string $html Additional HTML text to be shown. The name and size
 	 *   of the image are always shown.
-	 * @param string $alt Alt text for the image
+	 * @param string|null $alt Alt text for the image, or null to omit
 	 * @param string $link Override image link (optional)
 	 * @param array $handlerOpts Array of options for image handler (aka page number)
 	 * @param int $loading Sets loading attribute of the underlying <img> (optional)
+	 * @param ?array $imageOptions To supercede the $link param
 	 */
 	public function insert(
 			$title,
@@ -313,19 +336,20 @@ abstract class ImageGalleryBase extends ContextSource {
 			$alt = '',
 			$link = '',
 			$handlerOpts = [],
-			$loading = self::LOADING_DEFAULT
+			$loading = self::LOADING_DEFAULT,
+			?array $imageOptions = null
 		) {
 		if ( $title instanceof File ) {
 			// Old calling convention
 			$title = $title->getTitle();
 		}
-		array_unshift( $this->mImages, [ &$title, $html, $alt, $link, $handlerOpts, $loading ] );
+		array_unshift( $this->mImages, [ $title, $html, $alt, $link, $handlerOpts, $loading, $imageOptions ] );
 	}
 
 	/**
 	 * Returns the list of images this gallery contains
 	 * @return array[]
-	 * @phan-return array<int,array{0:Title,1:string,2:string,3:string,4:array}>
+	 * @phan-return array<int,array{0:Title,1:string,2:?string,3:string,4:array,5:int,6:?array}>
 	 */
 	public function getImages() {
 		return $this->mImages;
@@ -336,7 +360,7 @@ abstract class ImageGalleryBase extends ContextSource {
 	 * @return bool
 	 */
 	public function isEmpty() {
-		return empty( $this->mImages );
+		return $this->mImages === [];
 	}
 
 	/**
