@@ -21,8 +21,20 @@
  * @ingroup Installer
  */
 
+namespace MediaWiki\Installer;
+
+use Exception;
+use MediaWiki\Context\RequestContext;
+use MediaWiki\Html\Html;
+use MediaWiki\Installer\Task\TaskFactory;
+use MediaWiki\Installer\Task\TaskList;
+use MediaWiki\Installer\Task\TaskRunner;
 use MediaWiki\Languages\LanguageNameUtils;
 use MediaWiki\MediaWikiServices;
+use MediaWiki\Message\Message;
+use MediaWiki\Request\WebRequest;
+use MediaWiki\Status\Status;
+use Wikimedia\HtmlArmor\HtmlArmor;
 
 /**
  * Class for the core installer web interface.
@@ -138,9 +150,6 @@ class WebInstaller extends Installer {
 	 */
 	protected $currentPageName;
 
-	/**
-	 * @param WebRequest $request
-	 */
 	public function __construct( WebRequest $request ) {
 		parent::__construct();
 		$this->output = new WebInstallerOutput( $this );
@@ -174,7 +183,7 @@ class WebInstaller extends Installer {
 			return $this->session;
 		}
 
-		$isCSS = $this->request->getVal( 'css' );
+		$isCSS = $this->request->getCheck( 'css' );
 		if ( $isCSS ) {
 			$this->outputCss();
 			return $this->session;
@@ -186,29 +195,6 @@ class WebInstaller extends Installer {
 
 		$lowestUnhappy = $this->getLowestUnhappy();
 
-		# Special case for Creative Commons partner chooser box.
-		if ( $this->request->getVal( 'SubmitCC' ) ) {
-			/** @var WebInstallerOptions $page */
-			$page = $this->getPageByName( 'Options' );
-			'@phan-var WebInstallerOptions $page';
-			$this->output->useShortHeader();
-			$this->output->allowFrames();
-			$page->submitCC();
-
-			return $this->finish();
-		}
-
-		if ( $this->request->getVal( 'ShowCC' ) ) {
-			/** @var WebInstallerOptions $page */
-			$page = $this->getPageByName( 'Options' );
-			'@phan-var WebInstallerOptions $page';
-			$this->output->useShortHeader();
-			$this->output->allowFrames();
-			$this->output->addHTML( $page->getCCDoneBox() );
-
-			return $this->finish();
-		}
-
 		# Get the page name.
 		$pageName = $this->request->getVal( 'page', '' );
 
@@ -382,12 +368,6 @@ class WebInstaller extends Installer {
 		] ) );
 	}
 
-	/**
-	 * Show an error message in a box. Parameters are like wfMessage(), or
-	 * alternatively, pass a Message object in.
-	 * @param string|Message $msg
-	 * @param mixed ...$params
-	 */
 	public function showError( $msg, ...$params ) {
 		if ( !( $msg instanceof Message ) ) {
 			$msg = wfMessage(
@@ -460,7 +440,7 @@ class WebInstaller extends Installer {
 	 * @return WebInstallerPage
 	 */
 	public function getPageByName( $pageName ) {
-		$pageClass = 'WebInstaller' . $pageName;
+		$pageClass = 'MediaWiki\\Installer\\WebInstaller' . $pageName;
 
 		return new $pageClass( $this );
 	}
@@ -521,12 +501,12 @@ class WebInstaller extends Installer {
 	 * @return-taint none It can only return a known-good code.
 	 */
 	public function getAcceptLanguage() {
-		global $wgLanguageCode, $wgRequest;
+		global $wgLanguageCode;
 
 		$mwLanguages = MediaWikiServices::getInstance()
 			->getLanguageNameUtils()
 			->getLanguageNames( LanguageNameUtils::AUTONYMS, LanguageNameUtils::SUPPORTED );
-		$headerLanguages = array_keys( $wgRequest->getAcceptLang() );
+		$headerLanguages = array_keys( $this->request->getAcceptLang() );
 
 		foreach ( $headerLanguages as $lang ) {
 			if ( isset( $mwLanguages[$lang] ) ) {
@@ -545,7 +525,8 @@ class WebInstaller extends Installer {
 	private function startPageWrapper( $currentPageName ) {
 		$s = "<div class=\"config-page-wrapper\">\n";
 		$s .= "<div class=\"config-page\">\n";
-		$s .= "<div class=\"config-page-list\"><ul>\n";
+		$s .= "<div class=\"config-page-list cdx-card\"><span class=\"cdx-card__text\">";
+		$s .= "<span class=\"cdx-card__text__description\"><ul>\n";
 		$lastHappy = -1;
 
 		foreach ( $this->pageSequence as $id => $pageName ) {
@@ -564,7 +545,7 @@ class WebInstaller extends Installer {
 		$s .= "</ul><br/><ul>\n";
 		$s .= $this->getPageListItem( 'Restart', true, $currentPageName );
 		// End list pane
-		$s .= "</ul></div>\n";
+		$s .= "</ul></span></span></div>\n";
 
 		// Messages:
 		// config-page-language, config-page-welcome, config-page-dbconnect, config-page-upgrade,
@@ -644,81 +625,78 @@ class WebInstaller extends Installer {
 			"</div>" );
 	}
 
-	/**
-	 * Get HTML for an information message box with an icon.
-	 *
-	 * @param string|HtmlArmor $text Wikitext to be parsed (from Message::plain) or raw HTML.
-	 * @param string|false $icon Icon name, file in mw-config/images. Default: false
-	 * @param string $class Additional class name to add to the wrapper div. Default: Empty string.
-	 * @return string HTML
-	 */
-	public function getInfoBox( $text, $icon = false, $class = '' ) {
-		$html = ( $text instanceof HtmlArmor ) ?
-			HtmlArmor::getHtml( $text ) :
-			$this->parse( $text, true );
-		$icon = ( !$icon ) ?
-			'images/info-32.png' :
-			'images/' . $icon;
-		$alt = wfMessage( 'config-information' )->text();
-
-		return self::infoBox( $html, $icon, $alt, $class );
-	}
-
 	/**
 	 * Get small text indented help for a preceding form field.
 	 * Parameters like wfMessage().
 	 *
-	 * @param string $msg
-	 * @param mixed ...$args
+	 * @param string $msg Message key
+	 * @param string|int|float ...$params Message parameters
 	 * @return string HTML
 	 * @return-taint escaped
 	 */
-	public function getHelpBox( $msg, ...$args ) {
-		$args = array_map( 'htmlspecialchars', $args );
-		$text = wfMessage( $msg, $args )->useDatabase( false )->plain();
+	public function getHelpBox( $msg, ...$params ) {
+		$params = array_map( 'htmlspecialchars', $params );
+		$text = wfMessage( $msg, $params )->useDatabase( false )->plain();
 		$html = $this->parse( $text, true );
-		$id = 'helpBox-' . $this->helpBoxId++;
 
 		return "<div class=\"config-help-field-container\">\n" .
-			"<input type=\"checkbox\" class=\"config-help-field-checkbox\" id=\"$id\" />" .
-			"<label class=\"config-help-field-hint\" for=\"$id\" title=\"" .
-			wfMessage( 'config-help-tooltip' )->escaped() . "\">" .
-			wfMessage( 'config-help' )->escaped() . "</label>\n" .
-			"<div class=\"config-help-field-data\">" . $html . "</div>\n" .
+			"<a class=\"config-help-field-hint\" title=\"" .
+			wfMessage( 'config-help-tooltip' )->escaped() . "\">ℹ️ " .
+			wfMessage( 'config-help' )->escaped() . "</a>\n" .
+			"<div class=\"config-help-field-content config-help-field-content-hidden " .
+			"cdx-message cdx-message--block cdx-message--notice\" style=\"margin: 10px\">" .
+			"<div class=\"cdx-message__content\">" . $html . "</div></div>\n" .
 			"</div>\n";
 	}
 
 	/**
-	 * Output a help box.
-	 * @param string $msg Key for wfMessage()
-	 * @param mixed ...$params
-	 */
-	public function showHelpBox( $msg, ...$params ) {
-		$html = $this->getHelpBox( $msg, ...$params );
-		$this->output->addHTML( $html );
-	}
-
-	/**
-	 * Show a short informational message.
-	 * Output looks like a list.
+	 * Get HTML for an information message box.
 	 *
-	 * @param string $msg
-	 * @param mixed ...$params
+	 * @param string|HtmlArmor $text Wikitext to be parsed (from Message::plain) or raw HTML.
+	 * @return string HTML
 	 */
-	public function showMessage( $msg, ...$params ) {
-		$html = '<div class="config-message">' .
+	public function getInfoBox( $text ) {
+		$html = ( $text instanceof HtmlArmor ) ?
+			HtmlArmor::getHtml( $text ) :
+			$this->parse( $text, true );
+		return '<div class="cdx-message cdx-message--block cdx-message--notice">' .
+			'<span class="cdx-message__icon"></span><div class="cdx-message__content">' .
+			'<p><strong>' . wfMessage( 'config-information' )->escaped() . '</strong></p>' .
+			$html .
+			"</div></div>\n";
+	}
+
+	public function showSuccess( $msg, ...$params ) {
+		$html = '<div class="cdx-message cdx-message--block cdx-message--success">' .
+			'<span class="cdx-message__icon"></span><div class="cdx-message__content">' .
 			$this->parse( wfMessage( $msg, $params )->useDatabase( false )->plain() ) .
-			"</div>\n";
+			"</div></div>\n";
+		$this->output->addHTML( $html );
+	}
+
+	public function showMessage( $msg, ...$params ) {
+		$html = '<div class="cdx-message cdx-message--block cdx-message--notice">' .
+			'<span class="cdx-message__icon"></span><div class="cdx-message__content">' .
+			$this->parse( wfMessage( $msg, $params )->useDatabase( false )->plain() ) .
+			"</div></div>\n";
+		$this->output->addHTML( $html );
+	}
+
+	public function showWarning( $msg, ...$params ) {
+		$html = '<div class="cdx-message cdx-message--block cdx-message--warning">' .
+			'<span class="cdx-message__icon"></span><div class="cdx-message__content">' .
+			$this->parse( wfMessage( $msg, $params )->useDatabase( false )->plain() ) .
+			"</div></div>\n";
 		$this->output->addHTML( $html );
 	}
 
-	/**
-	 * @param Status $status
-	 */
 	public function showStatusMessage( Status $status ) {
-		$errors = array_merge( $status->getErrorsArray(), $status->getWarningsArray() );
-		foreach ( $errors as $error ) {
-			$this->showMessage( ...$error );
+		// Show errors at the top in web installer to make them easier to notice
+		foreach ( $status->getMessages( 'error' ) as $msg ) {
+			$this->showWarning( $msg );
+		}
+		foreach ( $status->getMessages( 'warning' ) as $msg ) {
+			$this->showWarning( $msg );
 		}
 	}
 
@@ -748,7 +726,7 @@ class WebInstaller extends Installer {
 
 		return "<div class=\"config-block\">\n" .
 			"  <div class=\"config-block-label\">\n" .
-			Xml::tags( 'label',
+			Html::rawElement( 'label',
 				$attributes,
 				$labelText
 			) . "\n" .
@@ -794,16 +772,18 @@ class WebInstaller extends Installer {
 		return $this->label(
 			$params['label'],
 			$params['controlName'],
-			Xml::input(
+			"<div class=\"cdx-text-input\">" .
+			Html::input(
 				$params['controlName'],
-				30, // intended to be overridden by CSS
 				$params['value'],
+				$params['attribs']['type'] ?? 'text',
 				$params['attribs'] + [
 					'id' => $params['controlName'],
-					'class' => 'config-input-text',
+					'size' => 30, // intended to be overridden by CSS
+					'class' => 'cdx-text-input__input',
 					'tabindex' => $this->nextTabIndex()
 				]
-			),
+			) . "</div>",
 			$params['help']
 		);
 	}
@@ -841,13 +821,13 @@ class WebInstaller extends Installer {
 		return $this->label(
 			$params['label'],
 			$params['controlName'],
-			Xml::textarea(
+			Html::textarea(
 				$params['controlName'],
 				$params['value'],
-				30,
-				5,
 				$params['attribs'] + [
 					'id' => $params['controlName'],
+					'cols' => 30,
+					'rows' => 5,
 					'class' => 'config-input-text',
 					'tabindex' => $this->nextTabIndex()
 				]
@@ -922,23 +902,30 @@ class WebInstaller extends Installer {
 			$params['labelAttribs'] = [];
 		}
 		$labelText = $params['rawtext'] ?? $this->parse( wfMessage( $params['label'] )->plain() );
+		$labelText = '<span class="cdx-label__label__text"> ' . $labelText . '</span>';
+		Html::addClass( $params['attribs']['class'], 'cdx-checkbox__input' );
+		Html::addClass( $params['labelAttribs']['class'], 'cdx-label__label' );
 
-		return "<div class=\"config-input-check\">\n" .
-			$params['help'] .
+		return "<div class=\"cdx-checkbox\" style=\"margin-top: 12px; margin-bottom: 2px;\">" .
+			"<div class=\"cdx-checkbox__wrapper\">\n" .
+			Html::check(
+				$params['controlName'],
+				$params['value'],
+				$params['attribs'] + [
+					'id' => $params['controlName'],
+					'tabindex' => $this->nextTabIndex()
+				]
+			) .
+			"<span class=\"cdx-checkbox__icon\"></span>" .
+			"<div class=\"cdx-checkbox__label cdx-label\">" .
 			Html::rawElement(
 				'label',
-				$params['labelAttribs'],
-				Xml::check(
-					$params['controlName'],
-					$params['value'],
-					$params['attribs'] + [
-						'id' => $params['controlName'],
-						'tabindex' => $this->nextTabIndex(),
-					]
+				$params['labelAttribs'] + [
+					'for' => $params['controlName']
+				],
+				$labelText
 				) .
-				$labelText . "\n"
-				) .
-			"</div>\n";
+			"</div></div></div>\n" . $params['help'];
 	}
 
 	/**
@@ -974,11 +961,10 @@ class WebInstaller extends Installer {
 			$params['help'] = "";
 		}
 
-		$s = "<ul>\n";
-		foreach ( $items as $value => $item ) {
-			$s .= "<li>$item</li>\n";
+		$s = "";
+		foreach ( $items as $item ) {
+			$s .= "$item\n";
 		}
-		$s .= "</ul>\n";
 
 		return $this->label( $label, $params['controlName'], $s, $params['help'] );
 	}
@@ -1018,15 +1004,24 @@ class WebInstaller extends Installer {
 			$id = $params['controlName'] . '_' . $value;
 			$itemAttribs['id'] = $id;
 			$itemAttribs['tabindex'] = $this->nextTabIndex();
+			Html::addClass( $itemAttribs['class'], 'cdx-radio__input' );
 
+			$radioText = $this->parse(
+				isset( $params['itemLabels'] ) ?
+					wfMessage( $params['itemLabels'][$value] )->plain() :
+					wfMessage( $params['itemLabelPrefix'] . strtolower( $value ) )->plain()
+			);
 			$items[$value] =
-				Xml::radio( $params['controlName'], $value, $checked, $itemAttribs ) .
-				"\u{00A0}" .
-				Xml::tags( 'label', [ 'for' => $id ], $this->parse(
-					isset( $params['itemLabels'] ) ?
-						wfMessage( $params['itemLabels'][$value] )->plain() :
-						wfMessage( $params['itemLabelPrefix'] . strtolower( $value ) )->plain()
-				) );
+				'<span class="cdx-radio">' .
+				'<span class="cdx-radio__wrapper">' .
+				Html::radio( $params['controlName'], $checked, $itemAttribs + [ 'value' => $value ] ) .
+				'<span class="cdx-radio__icon"></span>' .
+				'<span class="cdx-radio__label cdx-label">' .
+				Html::rawElement(
+					'label',
+					[ 'for' => $id, 'class' => 'cdx-label__label' ],
+					'<span class="cdx-label__label__text">' . $radioText . '</span>'
+				) . '</span></span></span>';
 		}
 
 		return $items;
@@ -1067,7 +1062,7 @@ class WebInstaller extends Installer {
 		foreach ( $varNames as $name ) {
 			$value = $this->request->getVal( $prefix . $name );
 			// T32524, do not trim passwords
-			if ( stripos( $name, 'password' ) === false ) {
+			if ( $value !== null && stripos( $name, 'password' ) === false ) {
 				$value = trim( $value );
 			}
 			$newValues[$name] = $value;
@@ -1116,21 +1111,6 @@ class WebInstaller extends Installer {
 		);
 	}
 
-	/**
-	 * Helper for "Download LocalSettings" link.
-	 *
-	 * @internal For use in WebInstallerComplete class
-	 * @return string Html for download link
-	 */
-	public function makeDownloadLinkHtml() {
-		$anchor = Html::rawElement( 'a',
-			[ 'href' => $this->getUrl( [ 'localsettings' => 1 ] ) ],
-			wfMessage( 'config-download-localsettings' )->parse()
-		);
-
-		return Html::rawElement( 'div', [ 'class' => 'config-download-link' ], $anchor );
-	}
-
 	/**
 	 * If the software package wants the LocalSettings.php file
 	 * to be placed in a specific location, override this function
@@ -1166,8 +1146,7 @@ class WebInstaller extends Installer {
 		return parent::envCheckPath();
 	}
 
-	public function envPrepPath() {
-		parent::envPrepPath();
+	protected function detectWebPaths() {
 		// PHP_SELF isn't available sometimes, such as when PHP is CGI but
 		// cgi.fix_pathinfo is disabled. In that case, fall back to SCRIPT_NAME
 		// to get the path to the current script... hopefully it's reliable. SIGH
@@ -1180,16 +1159,19 @@ class WebInstaller extends Installer {
 		if ( $path !== false ) {
 			$scriptPath = preg_replace( '{^(.*)/(mw-)?config.*$}', '$1', $path );
 
-			$this->setVar( 'wgScriptPath', "$scriptPath" );
-			// Update variables set from Setup.php that are derived from wgScriptPath
-			$this->setVar( 'wgScript', "$scriptPath/index.php" );
-			$this->setVar( 'wgLoadScript', "$scriptPath/load.php" );
-			$this->setVar( 'wgStylePath', "$scriptPath/skins" );
-			$this->setVar( 'wgLocalStylePath', "$scriptPath/skins" );
-			$this->setVar( 'wgExtensionAssetsPath', "$scriptPath/extensions" );
-			$this->setVar( 'wgUploadPath', "$scriptPath/images" );
-			$this->setVar( 'wgResourceBasePath', "$scriptPath" );
+			return [
+				'wgScriptPath' => "$scriptPath",
+				// Update variables set from Setup.php that are derived from wgScriptPath
+				'wgScript' => "$scriptPath/index.php",
+				'wgLoadScript' => "$scriptPath/load.php",
+				'wgStylePath' => "$scriptPath/skins",
+				'wgLocalStylePath' => "$scriptPath/skins",
+				'wgExtensionAssetsPath' => "$scriptPath/extensions",
+				'wgUploadPath' => "$scriptPath/images",
+				'wgResourceBasePath' => "$scriptPath",
+			];
 		}
+		return [];
 	}
 
 	/**
@@ -1202,6 +1184,13 @@ class WebInstaller extends Installer {
 		return WebRequest::detectServer( $assumeProxiesUseDefaultProtocolPorts );
 	}
 
+	/**
+	 * @return string
+	 */
+	public function getDefaultServer() {
+		return $this->envGetDefaultServer();
+	}
+
 	/**
 	 * Actually output LocalSettings.php for download
 	 */
@@ -1235,32 +1224,48 @@ class WebInstaller extends Installer {
 	}
 
 	/**
-	 * Get HTML for an information message box with an icon.
+	 * Determine whether the current database needs to be upgraded, i.e. whether
+	 * it already has MediaWiki tables.
 	 *
-	 * @since 1.36
-	 * @param string $rawHtml HTML
-	 * @param string $icon Path to icon file (used as 'src' attribute)
-	 * @param string $alt Alternate text for the icon
-	 * @param string $class Additional class name to add to the wrapper div
-	 * @return string HTML
+	 * @return bool
 	 */
-	protected static function infoBox( $rawHtml, $icon, $alt, $class = '' ) {
-		$s = Html::openElement( 'div', [ 'class' => 'mw-installer-box-left' ] ) .
-				Html::element( 'img',
-					[
-						'src' => $icon,
-						'alt' => $alt,
-					]
-				) .
-				Html::closeElement( 'div' );
+	public function needsUpgrade() {
+		return $this->getDBInstaller()->needsUpgrade();
+	}
 
-		$s .= Html::openElement( 'div', [ 'class' => 'mw-installer-box-right' ] ) .
-				$rawHtml .
-				Html::closeElement( 'div' );
-		$s .= Html::element( 'div', [ 'style' => 'clear: left;' ], ' ' );
+	/**
+	 * Perform database upgrades
+	 *
+	 * @return bool
+	 */
+	public function doUpgrade() {
+		$dbInstaller = $this->getDBInstaller();
+		$dbInstaller->preUpgrade();
 
-		return Html::warningBox( $s, $class )
-			. Html::element( 'div', [ 'style' => 'clear: left;' ], ' ' );
+		$taskList = new TaskList;
+		$taskFactory = $this->getTaskFactory();
+		$taskFactory->registerWebUpgradeTasks( $taskList );
+		$taskRunner = new TaskRunner( $taskList, $taskFactory, TaskFactory::PROFILE_WEB_UPGRADE );
+
+		ob_start( [ $this, 'outputHandler' ] );
+		try {
+			$status = $taskRunner->execute();
+			$ret = $status->isOK();
+
+			$this->setVar( '_UpgradeDone', true );
+		} catch ( Exception $e ) {
+			// TODO: Should this use MWExceptionRenderer?
+			echo "\nAn error occurred:\n";
+			echo $e->getMessage();
+			$ret = false;
+		}
+		ob_end_flush();
+
+		return $ret;
+	}
+
+	public function outputHandler( $string ) {
+		return htmlspecialchars( $string );
 	}
 
 }
