@@ -20,10 +20,14 @@
  * @file
  */
 
+namespace Wikimedia\Http;
+
+use InvalidArgumentException;
 use MediaWiki\MediaWikiServices;
 use Psr\Log\LoggerAwareInterface;
 use Psr\Log\LoggerInterface;
 use Psr\Log\NullLogger;
+use RuntimeException;
 
 /**
  * Class to handle multiple HTTP requests
@@ -57,9 +61,9 @@ class MultiHttpClient implements LoggerAwareInterface {
 	private const SENSITIVE_HEADERS = '/(^|-|_)(authorization|auth|password|cookie)($|-|_)/';
 	/**
 	 * @phpcs:ignore MediaWiki.Commenting.PropertyDocumentation.ObjectTypeHintVar
-	 * @var resource|object curl_multi_init() handle
+	 * @var resource|object|null curl_multi_init() handle, initialized in getCurlMulti()
 	 */
-	protected $cmh;
+	protected $cmh = null;
 	/** @var string|null SSL certificates path */
 	protected $caBundlePath;
 	/** @var float */
@@ -74,22 +78,25 @@ class MultiHttpClient implements LoggerAwareInterface {
 	protected $usePipelining = false;
 	/** @var int */
 	protected $maxConnsPerHost = 50;
-	/** @var string|null proxy */
+	/** @var string|null */
 	protected $proxy;
 	/** @var string|false */
 	protected $localProxy = false;
 	/** @var string[] */
 	protected $localVirtualHosts = [];
 	/** @var string */
-	protected $userAgent = 'wikimedia/multi-http-client v1.0';
+	protected $userAgent = 'wikimedia/multi-http-client v1.1';
 	/** @var LoggerInterface */
 	protected $logger;
+	protected array $headers = [];
 
 	// In PHP 7 due to https://bugs.php.net/bug.php?id=76480 the request/connect
 	// timeouts are periodically polled instead of being accurately respected.
 	// The select timeout is set to the minimum timeout multiplied by this factor.
 	private const TIMEOUT_ACCURACY_FACTOR = 0.1;
 
+	private ?TelemetryHeadersInterface $telemetry = null;
+
 	/**
 	 * Since 1.35, callers should use HttpRequestFactory::createMultiClient() to get
 	 * a client object with appropriately configured timeouts instead of constructing
@@ -108,19 +115,21 @@ class MultiHttpClient implements LoggerAwareInterface {
 	 *   - userAgent         : The User-Agent header value to send
 	 *   - logger            : a \Psr\Log\LoggerInterface instance for debug logging
 	 *   - caBundlePath      : path to specific Certificate Authority bundle (if any)
-	 * @throws Exception
+	 *   - headers           : an array of default headers to send with every request
+	 *   - telemetry         : a \Wikimedia\Http\RequestTelemetry instance to track telemetry data
+	 * @throws \Exception
 	 */
 	public function __construct( array $options ) {
 		if ( isset( $options['caBundlePath'] ) ) {
 			$this->caBundlePath = $options['caBundlePath'];
 			if ( !file_exists( $this->caBundlePath ) ) {
-				throw new Exception( "Cannot find CA bundle: " . $this->caBundlePath );
+				throw new InvalidArgumentException( "Cannot find CA bundle: " . $this->caBundlePath );
 			}
 		}
 		static $opts = [
 			'connTimeout', 'maxConnTimeout', 'reqTimeout', 'maxReqTimeout',
 			'usePipelining', 'maxConnsPerHost', 'proxy', 'userAgent', 'logger',
-			'localProxy', 'localVirtualHosts',
+			'localProxy', 'localVirtualHosts', 'headers', 'telemetry'
 		];
 		foreach ( $opts as $key ) {
 			if ( isset( $options[$key] ) ) {
@@ -151,10 +160,11 @@ class MultiHttpClient implements LoggerAwareInterface {
 	 *   - maxConnsPerHost : maximum number of concurrent connections (per host)
 	 *   - httpVersion     : One of 'v1.0', 'v1.1', 'v2' or 'v2.0'. Leave empty to use
 	 *                       PHP/curl's default
+	 * @param string $caller The method making this request, for attribution in logs
 	 * @return array Response array for request
 	 */
-	public function run( array $req, array $opts = [] ) {
-		return $this->runMulti( [ $req ], $opts )[0]['response'];
+	public function run( array $req, array $opts = [], string $caller = __METHOD__ ) {
+		return $this->runMulti( [ $req ], $opts, $caller )[0]['response'];
 	}
 
 	/**
@@ -185,10 +195,11 @@ class MultiHttpClient implements LoggerAwareInterface {
 	 *   - maxConnsPerHost : maximum number of concurrent connections (per host)
 	 *   - httpVersion     : One of 'v1.0', 'v1.1', 'v2' or 'v2.0'. Leave empty to use
 	 *                       PHP/curl's default
+	 * @param string $caller The method making these requests, for attribution in logs
 	 * @return array[] $reqs With response array populated for each
-	 * @throws Exception
+	 * @throws \Exception
 	 */
-	public function runMulti( array $reqs, array $opts = [] ) {
+	public function runMulti( array $reqs, array $opts = [], string $caller = __METHOD__ ) {
 		$this->normalizeRequests( $reqs );
 		$opts += [ 'connTimeout' => $this->connTimeout, 'reqTimeout' => $this->reqTimeout ];
 
@@ -214,7 +225,7 @@ class MultiHttpClient implements LoggerAwareInterface {
 				default:
 					$opts['httpVersion'] = CURL_HTTP_VERSION_NONE;
 			}
-			return $this->runMultiCurl( $reqs, $opts );
+			return $this->runMultiCurl( $reqs, $opts, $caller );
 		} else {
 			# TODO: Add handling for httpVersion option
 			return $this->runMultiHttp( $reqs, $opts );
@@ -245,11 +256,12 @@ class MultiHttpClient implements LoggerAwareInterface {
 	 *   - maxConnsPerHost : maximum number of concurrent connections (per host)
 	 *   - httpVersion:    : HTTP version to use
 	 * @phan-param array{connTimeout?:int,reqTimeout?:int,usePipelining?:bool,maxConnsPerHost?:int} $opts
+	 * @param string $caller The method making these requests, for attribution in logs
 	 * @return array $reqs With response array populated for each
-	 * @throws Exception
+	 * @throws \Exception
 	 * @suppress PhanTypeInvalidDimOffset
 	 */
-	private function runMultiCurl( array $reqs, array $opts ) {
+	private function runMultiCurl( array $reqs, array $opts, string $caller = __METHOD__ ) {
 		$chm = $this->getCurlMulti( $opts );
 
 		$selectTimeout = $this->getSelectTimeout( $opts );
@@ -267,22 +279,40 @@ class MultiHttpClient implements LoggerAwareInterface {
 		$active = null; // handles still being processed
 		do {
 			// Do any available work...
-			do {
-				$mrc = curl_multi_exec( $chm, $active );
-				$info = curl_multi_info_read( $chm );
-				if ( $info !== false ) {
-					// Note: cast to integer even works on PHP 8.0+ despite the
-					// handle being an object not a resource, because CurlHandle
-					// has a backwards-compatible cast_object handler.
-					$infos[(int)$info['handle']] = $info;
-				}
-			} while ( $mrc == CURLM_CALL_MULTI_PERFORM );
-			// Wait (if possible) for available work...
-			if ( $active > 0 && $mrc == CURLM_OK && curl_multi_select( $chm, $selectTimeout ) == -1 ) {
-				// PHP bug 63411; https://curl.haxx.se/libcurl/c/curl_multi_fdset.html
-				usleep( 5000 ); // 5ms
+			$mrc = curl_multi_exec( $chm, $active );
+
+			if ( $mrc !== CURLM_OK ) {
+				$error = curl_multi_strerror( $mrc );
+				$this->logger->error( 'curl_multi_exec() failed: {error}', [
+					'error' => $error,
+					'exception' => new RuntimeException(),
+					'method' => $caller,
+				] );
+				break;
 			}
-		} while ( $active > 0 && $mrc == CURLM_OK );
+
+			// Wait (if possible) for available work...
+			if ( $active > 0 && curl_multi_select( $chm, $selectTimeout ) === -1 ) {
+				$errno = curl_multi_errno( $chm );
+				$error = curl_multi_strerror( $errno );
+				$this->logger->error( 'curl_multi_select() failed: {error}', [
+					'error' => $error,
+					'exception' => new RuntimeException(),
+					'method' => $caller,
+				] );
+			}
+		} while ( $active > 0 );
+
+		$queuedMessages = null;
+		do {
+			$info = curl_multi_info_read( $chm, $queuedMessages );
+			if ( $info !== false && $info['msg'] === CURLMSG_DONE ) {
+				// Note: cast to integer even works on PHP 8.0+ despite the
+				// handle being an object not a resource, because CurlHandle
+				// has a backwards-compatible cast_object handler.
+				$infos[(int)$info['handle']] = $info;
+			}
+		} while ( $queuedMessages > 0 );
 
 		// Remove all of the added cURL handles and check for errors...
 		foreach ( $reqs as $index => &$req ) {
@@ -297,8 +327,12 @@ class MultiHttpClient implements LoggerAwareInterface {
 					if ( function_exists( 'curl_strerror' ) ) {
 						$req['response']['error'] .= " " . curl_strerror( $errno );
 					}
-					$this->logger->warning( "Error fetching URL \"{$req['url']}\": " .
-						$req['response']['error'] );
+					$this->logger->error( 'Error fetching URL "{url}": {error}', [
+						'url' => $req['url'],
+						'error' => $req['response']['error'],
+						'exception' => new RuntimeException(),
+						'method' => $caller,
+					] );
 				} else {
 					$this->logger->debug(
 						"HTTP complete: {method} {url} code={response_code} size={size} " .
@@ -342,14 +376,14 @@ class MultiHttpClient implements LoggerAwareInterface {
 	/**
 	 * @param array &$req HTTP request map
 	 * @phpcs:ignore Generic.Files.LineLength
-	 * @phan-param array{url:string,proxy?:?string,query:mixed,method:string,body:string|resource,headers:string[],stream?:resource,flags:array} $req
+	 * @phan-param array{url:string,proxy?:?string,query:mixed,method:string,body:string|resource,headers:array<string,string>,stream?:resource,flags:array} $req
 	 * @param array $opts
 	 *   - connTimeout : default connection timeout
 	 *   - reqTimeout : default request timeout
 	 *   - httpVersion: default HTTP version
 	 * @phpcs:ignore MediaWiki.Commenting.FunctionComment.ObjectTypeHintReturn
 	 * @return resource|object
-	 * @throws Exception
+	 * @throws \Exception
 	 */
 	protected function getCurlHandle( array &$req, array $opts ) {
 		$ch = curl_init();
@@ -388,7 +422,7 @@ class MultiHttpClient implements LoggerAwareInterface {
 				) {
 					curl_setopt( $ch, CURLOPT_UPLOAD, true );
 				} else {
-					throw new Exception( "Missing 'Content-Length' or 'Transfer-Encoding' header." );
+					throw new InvalidArgumentException( "Missing 'Content-Length' or 'Transfer-Encoding' header." );
 				}
 			} elseif ( $req['body'] !== '' ) {
 				$fp = fopen( "php://temp", "wb+" );
@@ -411,7 +445,7 @@ class MultiHttpClient implements LoggerAwareInterface {
 		} else {
 			// phpcs:ignore MediaWiki.Usage.ForbiddenFunctions.is_resource
 			if ( is_resource( $req['body'] ) || $req['body'] !== '' ) {
-				throw new Exception( "HTTP body specified for a non PUT/POST request." );
+				throw new InvalidArgumentException( "HTTP body specified for a non PUT/POST request." );
 			}
 			$req['headers']['content-length'] = 0;
 		}
@@ -422,8 +456,8 @@ class MultiHttpClient implements LoggerAwareInterface {
 
 		$headers = [];
 		foreach ( $req['headers'] as $name => $value ) {
-			if ( strpos( $name, ': ' ) ) {
-				throw new Exception( "Headers cannot have ':' in the name." );
+			if ( strpos( $name, ':' ) !== false ) {
+				throw new InvalidArgumentException( "Header name must not contain colon-space." );
 			}
 			$headers[] = $name . ': ' . trim( $value );
 		}
@@ -482,7 +516,7 @@ class MultiHttpClient implements LoggerAwareInterface {
 	 * @param array $opts
 	 * @phpcs:ignore MediaWiki.Commenting.FunctionComment.ObjectTypeHintReturn
 	 * @return resource|object
-	 * @throws Exception
+	 * @throws \Exception
 	 */
 	protected function getCurlMulti( array $opts ) {
 		if ( !$this->cmh ) {
@@ -546,13 +580,13 @@ class MultiHttpClient implements LoggerAwareInterface {
 	 * @todo Remove dependency on MediaWikiServices: rewrite using Guzzle T202352
 	 * @param array $reqs Map of HTTP request arrays
 	 * @phpcs:ignore Generic.Files.LineLength
-	 * @phan-param array<int,array{url:string,query:array,method:string,body:string,proxy?:?string,headers?:string[]}> $reqs
+	 * @phan-param array<int,array{url:string,query:array,method:string,body:string,headers:array<string,string>,proxy?:?string}> $reqs
 	 * @param array $opts
 	 *   - connTimeout     : connection timeout per request (seconds)
 	 *   - reqTimeout      : post-connection timeout per request (seconds)
 	 * @phan-param array{connTimeout:int,reqTimeout:int} $opts
 	 * @return array $reqs With response array populated for each
-	 * @throws Exception
+	 * @throws \Exception
 	 */
 	private function runMultiHttp( array $reqs, array $opts = [] ) {
 		$httpOptions = [
@@ -578,6 +612,9 @@ class MultiHttpClient implements LoggerAwareInterface {
 			$httpRequest = MediaWikiServices::getInstance()->getHttpRequestFactory()->create(
 				$url, $reqOptions, __METHOD__ );
 			$httpRequest->setLogger( $this->logger );
+			foreach ( $req['headers'] as $header => $value ) {
+				$httpRequest->setHeader( $header, $value );
+			}
 			$sv = $httpRequest->execute()->getStatusValue();
 
 			$respHeaders = array_map(
@@ -623,6 +660,19 @@ class MultiHttpClient implements LoggerAwareInterface {
 		return $reqs;
 	}
 
+	/**
+	 * Normalize headers array
+	 * @param array $headers
+	 * @return array
+	 */
+	private function normalizeHeaders( array $headers ): array {
+		$normalized = [];
+		foreach ( $headers as $name => $value ) {
+			$normalized[strtolower( $name )] = $value;
+		}
+		return $normalized;
+	}
+
 	/**
 	 * Normalize request information
 	 *
@@ -646,21 +696,22 @@ class MultiHttpClient implements LoggerAwareInterface {
 				unset( $req[1] );
 			}
 			if ( !isset( $req['method'] ) ) {
-				throw new Exception( "Request has no 'method' field set." );
+				throw new InvalidArgumentException( "Request has no 'method' field set." );
 			} elseif ( !isset( $req['url'] ) ) {
-				throw new Exception( "Request has no 'url' field set." );
+				throw new InvalidArgumentException( "Request has no 'url' field set." );
 			}
 			if ( $this->localProxy !== false && $this->isLocalURL( $req['url'] ) ) {
 				$this->useReverseProxy( $req, $this->localProxy );
 			}
 			$req['query'] ??= [];
-			$headers = []; // normalized headers
-			if ( isset( $req['headers'] ) ) {
-				foreach ( $req['headers'] as $name => $value ) {
-					$headers[strtolower( $name )] = $value;
-				}
-			}
-			$req['headers'] = $headers;
+			$req['headers'] = $this->normalizeHeaders(
+				array_merge(
+					$this->headers,
+					$this->telemetry ? $this->telemetry->getRequestHeaders() : [],
+					$req['headers'] ?? []
+				)
+			);
+
 			if ( !isset( $req['body'] ) ) {
 				$req['body'] = '';
 				$req['headers']['content-length'] = 0;
@@ -683,31 +734,78 @@ class MultiHttpClient implements LoggerAwareInterface {
 		}
 	}
 
-	private function useReverseProxy( array &$req, $proxy ) {
-		$parsedProxy = wfParseUrl( $proxy );
+	private function useReverseProxy( array &$req, string $proxy ) {
+		$parsedProxy = parse_url( $proxy );
 		if ( $parsedProxy === false ) {
-			throw new Exception( "Invalid reverseProxy configured: $proxy" );
+			throw new InvalidArgumentException( "Invalid reverseProxy configured: $proxy" );
 		}
-		$parsedUrl = wfParseUrl( $req['url'] );
+		$parsedUrl = parse_url( $req['url'] );
 		if ( $parsedUrl === false ) {
-			throw new Exception( "Invalid url specified: {$req['url']}" );
+			throw new InvalidArgumentException( "Invalid url specified: {$req['url']}" );
 		}
 		// Set the current host in the Host header
+		// @phan-suppress-next-line PhanTypePossiblyInvalidDimOffset
 		$req['headers']['Host'] = $parsedUrl['host'];
 		// Replace scheme, host and port in the request
+		// @phan-suppress-next-line PhanTypePossiblyInvalidDimOffset
 		$parsedUrl['scheme'] = $parsedProxy['scheme'];
+		// @phan-suppress-next-line PhanTypePossiblyInvalidDimOffset
 		$parsedUrl['host'] = $parsedProxy['host'];
 		if ( isset( $parsedProxy['port'] ) ) {
 			$parsedUrl['port'] = $parsedProxy['port'];
 		} else {
 			unset( $parsedUrl['port'] );
 		}
-		$req['url'] = wfAssembleUrl( $parsedUrl );
+		$req['url'] = self::assembleUrl( $parsedUrl );
 		// Explicitly disable use of another proxy by setting to false,
 		// since null will fallback to $this->proxy
 		$req['proxy'] = false;
 	}
 
+	/**
+	 * This is derived from MediaWiki\Utils\UrlUtils::assemble but changed to work
+	 * with parse_url's result so the delimiter is hardcoded.
+	 *
+	 * The basic structure used:
+	 * [scheme://][[user][:pass]@][host][:port][path][?query][#fragment]
+	 *
+	 * @param array $urlParts URL parts, as output from parse_url()
+	 * @return string URL assembled from its component parts
+	 */
+	private static function assembleUrl( array $urlParts ): string {
+		$result = isset( $urlParts['scheme'] ) ? $urlParts['scheme'] . '://' : '';
+
+		if ( isset( $urlParts['host'] ) ) {
+			if ( isset( $urlParts['user'] ) ) {
+				$result .= $urlParts['user'];
+				if ( isset( $urlParts['pass'] ) ) {
+					$result .= ':' . $urlParts['pass'];
+				}
+				$result .= '@';
+			}
+
+			$result .= $urlParts['host'];
+
+			if ( isset( $urlParts['port'] ) ) {
+				$result .= ':' . $urlParts['port'];
+			}
+		}
+
+		if ( isset( $urlParts['path'] ) ) {
+			$result .= $urlParts['path'];
+		}
+
+		if ( isset( $urlParts['query'] ) && $urlParts['query'] !== '' ) {
+			$result .= '?' . $urlParts['query'];
+		}
+
+		if ( isset( $urlParts['fragment'] ) ) {
+			$result .= '#' . $urlParts['fragment'];
+		}
+
+		return $result;
+	}
+
 	/**
 	 * Check if the URL can be served by localhost
 	 *
@@ -773,16 +871,18 @@ class MultiHttpClient implements LoggerAwareInterface {
 
 	/**
 	 * Register a logger
-	 *
-	 * @param LoggerInterface $logger
 	 */
-	public function setLogger( LoggerInterface $logger ) {
+	public function setLogger( LoggerInterface $logger ): void {
 		$this->logger = $logger;
 	}
 
 	public function __destruct() {
 		if ( $this->cmh ) {
 			curl_multi_close( $this->cmh );
+			$this->cmh = null;
 		}
 	}
+
 }
+/** @deprecated class alias since 1.43 */
+class_alias( MultiHttpClient::class, 'MultiHttpClient' );
