@@ -1,8 +1,10 @@
 <?php
 
 use MediaWiki\Config\Config;
+use MediaWiki\Config\ServiceOptions;
 use MediaWiki\HookContainer\HookContainer;
 use MediaWiki\HookContainer\HookRunner;
+use MediaWiki\Language\Language;
 use MediaWiki\MainConfigNames;
 use MediaWiki\User\Options\UserOptionsLookup;
 use MediaWiki\User\UserIdentity;
@@ -15,18 +17,19 @@ use MediaWiki\User\UserIdentity;
  */
 class SearchEngineConfig {
 
+	/** @internal For use by ServiceWiring.php ONLY */
+	public const CONSTRUCTOR_OPTIONS = [
+		MainConfigNames::NamespacesToBeSearchedDefault,
+		MainConfigNames::SearchTypeAlternatives,
+		MainConfigNames::SearchType,
+	];
+
 	/**
 	 * Config object from which the settings will be derived.
 	 * @var Config
 	 */
 	private $config;
 
-	/**
-	 * Current language
-	 * @var Language
-	 */
-	private $language;
-
 	/**
 	 * Search Engine Mappings
 	 *
@@ -37,42 +40,33 @@ class SearchEngineConfig {
 	 */
 	private $engineMappings;
 
-	/**
-	 * @var HookRunner
-	 */
-	private $hookRunner;
+	private ServiceOptions $options;
+	private Language $language;
+	private HookRunner $hookRunner;
+	private UserOptionsLookup $userOptionsLookup;
 
-	/**
-	 * @var UserOptionsLookup
-	 */
-	private $userOptionsLookup;
-
-	/**
-	 * @param Config $config
-	 * @param Language $lang
-	 * @param HookContainer $hookContainer
-	 * @param array $mappings
-	 * @param UserOptionsLookup $userOptionsLookup
-	 */
 	public function __construct(
-		Config $config,
-		Language $lang,
+		ServiceOptions $options,
+		Language $language,
 		HookContainer $hookContainer,
-		array $mappings,
+		array $engineMappings,
 		UserOptionsLookup $userOptionsLookup
 	) {
-		$this->config = $config;
-		$this->language = $lang;
-		$this->engineMappings = $mappings;
+		$options->assertRequiredOptions( self::CONSTRUCTOR_OPTIONS );
+		$this->options = $options;
+		$this->language = $language;
+		$this->engineMappings = $engineMappings;
 		$this->hookRunner = new HookRunner( $hookContainer );
 		$this->userOptionsLookup = $userOptionsLookup;
 	}
 
 	/**
 	 * Retrieve original config.
+	 * @deprecated since 1.43, use ServiceOptions instead with DI.
 	 * @return Config
 	 */
 	public function getConfig() {
+		wfDeprecated( __METHOD__, '1.43' );
 		return $this->config;
 	}
 
@@ -117,7 +111,7 @@ class SearchEngineConfig {
 	 * @return int[] Namespace IDs
 	 */
 	public function defaultNamespaces() {
-		return array_keys( $this->config->get( MainConfigNames::NamespacesToBeSearchedDefault ),
+		return array_keys( $this->options->get( MainConfigNames::NamespacesToBeSearchedDefault ),
 			true );
 	}
 
@@ -128,8 +122,8 @@ class SearchEngineConfig {
 	 * @return array
 	 */
 	public function getSearchTypes() {
-		$alternatives = $this->config->get( MainConfigNames::SearchTypeAlternatives ) ?: [];
-		array_unshift( $alternatives, $this->config->get( MainConfigNames::SearchType ) );
+		$alternatives = $this->options->get( MainConfigNames::SearchTypeAlternatives ) ?: [];
+		array_unshift( $alternatives, $this->options->get( MainConfigNames::SearchType ) );
 
 		return $alternatives;
 	}
@@ -140,7 +134,7 @@ class SearchEngineConfig {
 	 * @return string|null
 	 */
 	public function getSearchType() {
-		return $this->config->get( MainConfigNames::SearchType );
+		return $this->options->get( MainConfigNames::SearchType );
 	}
 
 	/**
