@@ -23,10 +23,15 @@
 
 namespace MediaWiki\Session;
 
+use BadMethodCallException;
+use LogicException;
+use MediaWiki\MainConfigNames;
 use MediaWiki\MediaWikiServices;
+use MediaWiki\Request\WebRequest;
+use MediaWiki\User\User;
+use MWRestrictions;
 use Psr\Log\LoggerInterface;
-use User;
-use WebRequest;
+use RuntimeException;
 
 /**
  * Manages data for an authenticated session
@@ -50,14 +55,13 @@ class Session implements \Countable, \Iterator, \ArrayAccess {
 	/** @var null|string[] Encryption algorithm to use */
 	private static $encryptionAlgorithm = null;
 
-	/** @var SessionBackend Session backend */
+	/** @var SessionBackend Session backend (can't be type-hinted, see DummySessionBackend in tests) */
 	private $backend;
 
 	/** @var int Session index */
 	private $index;
 
-	/** @var LoggerInterface */
-	private $logger;
+	private LoggerInterface $logger;
 
 	/**
 	 * @param SessionBackend $backend
@@ -168,9 +172,8 @@ class Session implements \Countable, \Iterator, \ArrayAccess {
 
 	/**
 	 * Returns the authenticated user for this session
-	 * @return User
 	 */
-	public function getUser() {
+	public function getUser(): User {
 		return $this->backend->getUser();
 	}
 
@@ -182,6 +185,15 @@ class Session implements \Countable, \Iterator, \ArrayAccess {
 		return $this->backend->getAllowedUserRights();
 	}
 
+	/**
+	 * Fetch any restrictions imposed on logins or actions when this
+	 * session is active.
+	 * @return MWRestrictions|null
+	 */
+	public function getRestrictions(): ?MWRestrictions {
+		return $this->backend->getRestrictions();
+	}
+
 	/**
 	 * Indicate whether the session user info can be changed
 	 * @return bool
@@ -265,7 +277,7 @@ class Session implements \Countable, \Iterator, \ArrayAccess {
 			$this->backend->dirty();
 		}
 		if ( $this->backend->canSetUser() ) {
-			$this->backend->setUser( new User );
+			$this->backend->setUser( MediaWikiServices::getInstance()->getUserFactory()->newAnonymous() );
 		}
 		$this->backend->save();
 	}
@@ -412,10 +424,8 @@ class Session implements \Countable, \Iterator, \ArrayAccess {
 	 */
 	private function getSecretKeys() {
 		$mainConfig = MediaWikiServices::getInstance()->getMainConfig();
-		$sessionSecret = $mainConfig->get( 'SessionSecret' );
-		$secretKey = $mainConfig->get( 'SecretKey' );
-		$sessionPbkdf2Iterations = $mainConfig->get( 'SessionPbkdf2Iterations' );
-		$wikiSecret = $sessionSecret ?: $secretKey;
+		$wikiSecret = $mainConfig->get( MainConfigNames::SessionSecret )
+			?: $mainConfig->get( MainConfigNames::SecretKey );
 		$userSecret = $this->get( 'wsSessionSecret', null );
 		if ( $userSecret === null ) {
 			$userSecret = \MWCryptRand::generateHex( 32 );
@@ -423,11 +433,11 @@ class Session implements \Countable, \Iterator, \ArrayAccess {
 		}
 		$iterations = $this->get( 'wsSessionPbkdf2Iterations', null );
 		if ( $iterations === null ) {
-			$iterations = $sessionPbkdf2Iterations;
+			$iterations = $mainConfig->get( MainConfigNames::SessionPbkdf2Iterations );
 			$this->set( 'wsSessionPbkdf2Iterations', $iterations );
 		}
 
-		$keymats = hash_pbkdf2( 'sha256', $wikiSecret, $userSecret, $iterations, 64, true );
+		$keymats = openssl_pbkdf2( $wikiSecret, $userSecret, 64, $iterations, 'sha256' );
 		return [
 			substr( $keymats, 0, 32 ),
 			substr( $keymats, 32, 32 ),
@@ -439,8 +449,6 @@ class Session implements \Countable, \Iterator, \ArrayAccess {
 	 * @return array
 	 */
 	private static function getEncryptionAlgorithm() {
-		$sessionInsecureSecrets = MediaWikiServices::getInstance()->getMainConfig()->get( 'SessionInsecureSecrets' );
-
 		if ( self::$encryptionAlgorithm === null ) {
 			if ( function_exists( 'openssl_encrypt' ) ) {
 				$methods = openssl_get_cipher_methods();
@@ -454,17 +462,8 @@ class Session implements \Countable, \Iterator, \ArrayAccess {
 				}
 			}
 
-			if ( $sessionInsecureSecrets ) {
-				// @todo: import a pure-PHP library for AES instead of this
-				self::$encryptionAlgorithm = [ 'insecure' ];
-				return self::$encryptionAlgorithm;
-			}
-
-			throw new \BadMethodCallException(
-				'Encryption is not available. You really should install the PHP OpenSSL extension. ' .
-				'But if you really can\'t and you\'re willing ' .
-				'to accept insecure storage of sensitive session data, set ' .
-				'$wgSessionInsecureSecrets = true in LocalSettings.php to make this exception go away.'
+			throw new BadMethodCallException(
+				'Encryption is not available. You need to install the PHP OpenSSL extension.'
 			);
 		}
 
@@ -480,14 +479,13 @@ class Session implements \Countable, \Iterator, \ArrayAccess {
 	 * @param mixed $value
 	 */
 	public function setSecret( $key, $value ) {
-		list( $encKey, $hmacKey ) = $this->getSecretKeys();
+		[ $encKey, $hmacKey ] = $this->getSecretKeys();
 		$serialized = serialize( $value );
 
 		// The code for encryption (with OpenSSL) and sealing is taken from
 		// Chris Steipp's OATHAuthUtils class in Extension::OATHAuth.
 
 		// Encrypt
-		// @todo: import a pure-PHP library for AES instead of doing $wgSessionInsecureSecrets
 		$iv = random_bytes( 16 );
 		$algorithm = self::getEncryptionAlgorithm();
 		switch ( $algorithm[0] ) {
@@ -497,13 +495,8 @@ class Session implements \Countable, \Iterator, \ArrayAccess {
 					throw new \UnexpectedValueException( 'Encryption failed: ' . openssl_error_string() );
 				}
 				break;
-			case 'insecure':
-				$ex = new \Exception( 'No encryption is available, storing data as plain text' );
-				$this->logger->warning( $ex->getMessage(), [ 'exception' => $ex ] );
-				$ciphertext = $serialized;
-				break;
 			default:
-				throw new \LogicException( 'invalid algorithm' );
+				throw new LogicException( 'invalid algorithm' );
 		}
 
 		// Seal
@@ -535,15 +528,15 @@ class Session implements \Countable, \Iterator, \ArrayAccess {
 		// Unseal and check
 		$pieces = explode( '.', $encrypted, 4 );
 		if ( count( $pieces ) !== 3 ) {
-			$ex = new \Exception( 'Invalid sealed-secret format' );
+			$ex = new RuntimeException( 'Invalid sealed-secret format' );
 			$this->logger->warning( $ex->getMessage(), [ 'exception' => $ex ] );
 			return $default;
 		}
-		list( $hmac, $iv, $ciphertext ) = $pieces;
-		list( $encKey, $hmacKey ) = $this->getSecretKeys();
+		[ $hmac, $iv, $ciphertext ] = $pieces;
+		[ $encKey, $hmacKey ] = $this->getSecretKeys();
 		$integCalc = hash_hmac( 'sha256', $iv . '.' . $ciphertext, $hmacKey, true );
 		if ( !hash_equals( $integCalc, base64_decode( $hmac ) ) ) {
-			$ex = new \Exception( 'Sealed secret has been tampered with, aborting.' );
+			$ex = new RuntimeException( 'Sealed secret has been tampered with, aborting.' );
 			$this->logger->warning( $ex->getMessage(), [ 'exception' => $ex ] );
 			return $default;
 		}
@@ -555,18 +548,11 @@ class Session implements \Countable, \Iterator, \ArrayAccess {
 				$serialized = openssl_decrypt( base64_decode( $ciphertext ), $algorithm[1], $encKey,
 					OPENSSL_RAW_DATA, base64_decode( $iv ) );
 				if ( $serialized === false ) {
-					$ex = new \Exception( 'Decyption failed: ' . openssl_error_string() );
+					$ex = new RuntimeException( 'Decyption failed: ' . openssl_error_string() );
 					$this->logger->debug( $ex->getMessage(), [ 'exception' => $ex ] );
 					return $default;
 				}
 				break;
-			case 'insecure':
-				$ex = new \Exception(
-					'No encryption is available, retrieving data that was stored as plain text'
-				);
-				$this->logger->warning( $ex->getMessage(), [ 'exception' => $ex ] );
-				$serialized = base64_decode( $ciphertext );
-				break;
 			default:
 				throw new \LogicException( 'invalid algorithm' );
 		}
@@ -662,7 +648,7 @@ class Session implements \Countable, \Iterator, \ArrayAccess {
 	public function &offsetGet( $offset ) {
 		$data = &$this->backend->getData();
 		if ( !array_key_exists( $offset, $data ) ) {
-			$ex = new \Exception( "Undefined index (auto-adds to session with a null value): $offset" );
+			$ex = new LogicException( "Undefined index (auto-adds to session with a null value): $offset" );
 			$this->logger->debug( $ex->getMessage(), [ 'exception' => $ex ] );
 		}
 		return $data[$offset];
