@@ -35,6 +35,9 @@ require_once __DIR__ . '/../../includes/export/WikiExporter.php';
 use BaseDump;
 use Exception;
 use ExportProgressFilter;
+use MediaWiki\Exception\MWException;
+use MediaWiki\Exception\MWUnknownContentModelException;
+use MediaWiki\Revision\RevisionStore;
 use MediaWiki\Revision\SlotRecord;
 use MediaWiki\Settings\SettingsBuilder;
 use MediaWiki\Shell\Shell;
@@ -43,12 +46,9 @@ use MediaWiki\Storage\BlobStore;
 use MediaWiki\Storage\SqlBlobStore;
 use MediaWiki\WikiMap\WikiMap;
 use MediaWiki\Xml\Xml;
-use MWException;
-use MWUnknownContentModelException;
 use RuntimeException;
 use WikiExporter;
 use Wikimedia\AtEase\AtEase;
-use Wikimedia\Rdbms\IMaintainableDatabase;
 use XmlDumpWriter;
 
 /**
@@ -135,11 +135,6 @@ class TextPassDumper extends BackupDumper {
 	/** @var string[] */
 	protected $checkpointFiles = [];
 
-	/**
-	 * @var IMaintainableDatabase
-	 */
-	protected $db;
-
 	/**
 	 * @param array|null $args For backward compatibility
 	 */
@@ -194,6 +189,13 @@ TEXT
 		return $this->getServiceContainer()->getBlobStore();
 	}
 
+	/**
+	 * @return RevisionStore
+	 */
+	private function getRevisionStore() {
+		return $this->getServiceContainer()->getRevisionStore();
+	}
+
 	public function execute() {
 		$this->processOptions();
 		$this->dump( true );
@@ -240,56 +242,6 @@ TEXT
 		}
 	}
 
-	/**
-	 * Drop the database connection $this->db and try to get a new one.
-	 *
-	 * This function tries to get a /different/ connection if this is
-	 * possible. Hence, (if this is possible) it switches to a different
-	 * failover upon each call.
-	 *
-	 * This function resets $this->lb and closes all connections on it.
-	 *
-	 * @suppress PhanTypeObjectUnsetDeclaredProperty
-	 */
-	protected function rotateDb() {
-		// Cleaning up old connections
-		if ( isset( $this->lb ) ) {
-			$this->lb->closeAll( __METHOD__ );
-			unset( $this->lb );
-		}
-
-		if ( $this->forcedDb !== null ) {
-			$this->db = $this->forcedDb;
-
-			return;
-		}
-
-		if ( isset( $this->db ) && $this->db->isOpen() ) {
-			throw new RuntimeException( 'DB is set and has not been closed by the Load Balancer' );
-		}
-
-		unset( $this->db );
-
-		// Trying to set up new connection.
-		// We do /not/ retry upon failure, but delegate to encapsulating logic, to avoid
-		// individually retrying at different layers of code.
-
-		try {
-			$lbFactory = $this->getServiceContainer()->getDBLoadBalancerFactory();
-			$this->lb = $lbFactory->newMainLB();
-		} catch ( Exception $e ) {
-			throw new RuntimeException( __METHOD__
-				. " rotating DB failed to obtain new load balancer (" . $e->getMessage() . ")" );
-		}
-
-		try {
-			$this->db = $this->lb->getMaintenanceConnectionRef( DB_REPLICA, 'dump' );
-		} catch ( Exception $e ) {
-			throw new RuntimeException( __METHOD__
-				. " rotating DB failed to obtain new database (" . $e->getMessage() . ")" );
-		}
-	}
-
 	public function initProgress( $history = WikiExporter::FULL ) {
 		parent::initProgress();
 		$this->timeOfCheckpoint = $this->startTime;
@@ -304,20 +256,6 @@ TEXT
 
 		$this->initProgress( $this->history );
 
-		// We are trying to get an initial database connection to avoid that the
-		// first try of this request's first call to getText fails. However, if
-		// obtaining a good DB connection fails it's not a serious issue, as
-		// getText does retry upon failure and can start without having a working
-		// DB connection.
-		try {
-			$this->rotateDb();
-		} catch ( Exception $e ) {
-			// We do not even count this as failure. Just let eventual
-			// watchdogs know.
-			$this->progress( "Getting initial DB connection failed (" .
-				$e->getMessage() . ")" );
-		}
-
 		$this->egress = new ExportProgressFilter( $this->sink, $this );
 
 		// it would be nice to do it in the constructor, oh well. need egress set
@@ -336,7 +274,7 @@ TEXT
 		$this->report( true );
 	}
 
-	protected function processFileOpt( $opt ) {
+	protected function processFileOpt( string $opt ): string {
 		$split = explode( ':', $opt, 2 );
 		$val = $split[0];
 		$param = '';
@@ -438,7 +376,7 @@ TEXT
 		$this->timeExceeded = true;
 	}
 
-	private function checkIfTimeExceeded() {
+	private function checkIfTimeExceeded(): bool {
 		if ( $this->maxTimeAllowed
 			&& ( $this->lastTime - $this->timeOfCheckpoint > $this->maxTimeAllowed )
 		) {
@@ -613,6 +551,7 @@ TEXT
 		$prefetchNotTried = true; // Whether or not we already tried to get the text via prefetch.
 		$text = false; // The candidate for a good text. false if no proper value.
 		$failures = 0; // The number of times, this invocation of getText already failed.
+		$contentAddress = $id; // Where the content should be found
 
 		// The number of times getText failed without yielding a good text in between.
 		static $consecutiveFailedTextRetrievals = 0;
@@ -636,7 +575,7 @@ TEXT
 
 				// Trying to get prefetch, if it has not been tried before
 				// @phan-suppress-next-line PhanSuspiciousValueComparisonInLoop
-				if ( $text === false && isset( $this->prefetch ) && $prefetchNotTried ) {
+				if ( $text === false && $this->prefetch && $prefetchNotTried ) {
 					$prefetchNotTried = false;
 					$tryIsPrefetch = true;
 					$text = $this->prefetch->prefetch(
@@ -659,9 +598,9 @@ TEXT
 					// Fallback to asking the database
 					$tryIsPrefetch = false;
 					if ( $this->spawn ) {
-						$text = $this->getTextSpawned( $id );
+						$text = $this->getTextSpawned( $contentAddress );
 					} else {
-						$text = $this->getTextDb( $id );
+						$text = $this->getTextDb( $contentAddress );
 					}
 
 					if ( $text !== false && $model !== null ) {
@@ -679,7 +618,7 @@ TEXT
 				}
 
 				if ( $text === false ) {
-					throw new RuntimeException( "Generic error while obtaining text for id " . $id );
+					throw new RuntimeException( "Generic error while obtaining text for id " . $contentAddress );
 				}
 
 				// We received a good candidate for the text of $id via some method
@@ -697,9 +636,9 @@ TEXT
 				}
 
 				$text = false;
-				throw new RuntimeException( "Received text is unplausible for id " . $id );
+				throw new RuntimeException( "Received text is unplausible for id " . $contentAddress );
 			} catch ( Exception $e ) {
-				$msg = "getting/checking text " . $id . " failed (" . $e->getMessage()
+				$msg = "getting/checking text " . $contentAddress . " failed (" . $e->getMessage()
 					. ") for revision " . $this->thisRev;
 				if ( $failures + 1 < $this->maxFailures ) {
 					$msg .= " (Will retry " . ( $this->maxFailures - $failures - 1 ) . " more times)";
@@ -707,18 +646,41 @@ TEXT
 				$this->progress( $msg );
 			}
 
-			// Something went wrong; we did not a text that was plausible :(
+			// Something went wrong; we did not get a text that was plausible :(
 			$failures++;
 
+			if ( $contentAddress === $id && $this->thisRev && trim( $this->thisRole ) ) {
+				try {
+					// MediaWiki doesn't guarantee that content addresses are valid
+					// for any significant length of time. Try refreshing as the
+					// previously retrieved address may no longer be valid.
+					$revRecord = $this->getRevisionStore()->getRevisionById( (int)$this->thisRev );
+					if ( $revRecord !== null ) {
+						$refreshed = $revRecord->getSlot( trim( $this->thisRole ) )->getAddress();
+						if ( $contentAddress !== $refreshed ) {
+							$this->progress(
+								"Updated content address for rev {$this->thisRev} from "
+								. "{$contentAddress} to {$refreshed}"
+							);
+							$contentAddress = $refreshed;
+							// Skip sleeping if we updated the address
+							continue;
+						}
+					}
+				} catch ( Exception $e ) {
+					$this->progress(
+						"refreshing content address for revision {$this->thisRev} failed ({$e->getMessage()})"
+					);
+				}
+			}
+
 			// A failure in a prefetch hit does not warrant resetting db connection etc.
-			// @phan-suppress-next-line PhanPossiblyUndeclaredVariable Control flow is hard to understand here.
 			if ( !$tryIsPrefetch ) {
 				// After backing off for some time, we try to reboot the whole process as
 				// much as possible to not carry over failures from one part to the other
 				// parts
 				sleep( $this->failureTimeout );
 				try {
-					$this->rotateDb();
 					if ( $this->spawn ) {
 						$this->closeSpawn();
 						$this->openSpawn();
@@ -785,7 +747,7 @@ TEXT
 		return $text;
 	}
 
-	protected function openSpawn() {
+	protected function openSpawn(): bool {
 		global $IP;
 
 		$wiki = WikiMap::getCurrentWikiId();
@@ -846,7 +808,7 @@ TEXT
 		}
 		$this->spawnErr = false;
 		if ( $this->spawnProc ) {
-			pclose( $this->spawnProc );
+			proc_close( $this->spawnProc );
 		}
 		$this->spawnProc = false;
 		AtEase::restoreWarnings();
@@ -926,7 +888,7 @@ TEXT
 		return $normalized;
 	}
 
-	protected function startElement( $parser, $name, $attribs ) {
+	protected function startElement( $parser, string $name, array $attribs ) {
 		$this->checkpointJustWritten = false;
 
 		$this->clearOpenElement( null );
@@ -967,12 +929,12 @@ TEXT
 
 			unset( $attribs['id'] );
 			unset( $attribs['location'] );
-			if ( strlen( $text ) > 0 ) {
+			if ( $text !== '' ) {
 				$attribs['xml:space'] = 'preserve';
 			}
 
 			$this->openElement = [ $name, $attribs ];
-			if ( strlen( $text ) > 0 ) {
+			if ( $text !== '' ) {
 				$this->characterData( $parser, $text );
 			}
 		} else {
@@ -980,7 +942,7 @@ TEXT
 		}
 	}
 
-	protected function endElement( $parser, $name ) {
+	protected function endElement( $parser, string $name ) {
 		$this->checkpointJustWritten = false;
 
 		if ( $this->openElement ) {
@@ -1041,7 +1003,7 @@ TEXT
 		}
 	}
 
-	protected function characterData( $parser, $data ) {
+	protected function characterData( $parser, string $data ) {
 		$this->clearOpenElement( null );
 		if ( $this->lastName == "id" ) {
 			if ( $this->state == "revision" ) {
@@ -1073,14 +1035,14 @@ TEXT
 		$this->buffer .= htmlspecialchars( $data, ENT_COMPAT );
 	}
 
-	protected function clearOpenElement( $style ) {
+	protected function clearOpenElement( ?string $style ) {
 		if ( $this->openElement ) {
 			$this->buffer .= Xml::element( $this->openElement[0], $this->openElement[1], $style );
 			$this->openElement = false;
 		}
 	}
 
-	private function isValidTextId( $id ) {
+	private function isValidTextId( string $id ): bool {
 		if ( preg_match( '/:/', $id ) ) {
 			return $id !== 'tt:0';
 		} elseif ( preg_match( '/^\d+$/', $id ) ) {
