@@ -2,6 +2,7 @@
 
 namespace MediaWiki\Tests\Storage;
 
+use ArrayUtils;
 use DummyContentHandlerForTesting;
 use MediaWiki\CommentStore\CommentStoreComment;
 use MediaWiki\Config\ServiceOptions;
@@ -16,13 +17,18 @@ use MediaWiki\Deferred\DeferredUpdates;
 use MediaWiki\Deferred\LinksUpdate\LinksUpdate;
 use MediaWiki\Deferred\MWCallableUpdate;
 use MediaWiki\Edit\ParsoidRenderID;
+use MediaWiki\Logging\LogPage;
+use MediaWiki\Logging\ManualLogEntry;
 use MediaWiki\MainConfigNames;
 use MediaWiki\Message\Message;
+use MediaWiki\Page\Event\PageRevisionUpdatedEvent;
 use MediaWiki\Page\PageIdentity;
 use MediaWiki\Page\PageIdentityValue;
 use MediaWiki\Page\ParserOutputAccess;
+use MediaWiki\Page\WikiPage;
 use MediaWiki\Parser\ParserCacheFactory;
 use MediaWiki\Parser\ParserOptions;
+use MediaWiki\RecentChanges\RecentChange;
 use MediaWiki\Revision\MutableRevisionRecord;
 use MediaWiki\Revision\MutableRevisionSlots;
 use MediaWiki\Revision\RevisionRecord;
@@ -31,6 +37,7 @@ use MediaWiki\Storage\DerivedPageDataUpdater;
 use MediaWiki\Storage\EditResult;
 use MediaWiki\Storage\EditResultCache;
 use MediaWiki\Storage\RevisionSlotsUpdate;
+use MediaWiki\Tests\ExpectCallbackTrait;
 use MediaWiki\Title\Title;
 use MediaWiki\User\User;
 use MediaWiki\User\UserIdentity;
@@ -38,12 +45,12 @@ use MediaWiki\User\UserIdentityValue;
 use MediaWiki\Utils\MWTimestamp;
 use MediaWikiIntegrationTestCase;
 use MockTitleTrait;
+use PHPUnit\Framework\Assert;
 use PHPUnit\Framework\MockObject\MockObject;
 use Wikimedia\ObjectCache\BagOStuff;
 use Wikimedia\Rdbms\Platform\ISQLPlatform;
 use Wikimedia\TestingAccessWrapper;
 use Wikimedia\Timestamp\ConvertibleTimestamp;
-use WikiPage;
 
 /**
  * @group Database
@@ -52,6 +59,7 @@ use WikiPage;
  */
 class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 	use MockTitleTrait;
+	use ExpectCallbackTrait;
 
 	/**
 	 * @param string $title
@@ -127,6 +135,7 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 		}
 
 		$this->getDerivedPageDataUpdater( $page ); // flush cached instance after.
+		$this->runJobs( [ 'minJobs' => 0 ] ); // flush pending updates
 		return $rev;
 	}
 
@@ -440,6 +449,53 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 		$this->assertStringContainsString( 'second', $canonicalOutput->getRawText() );
 	}
 
+	/**
+	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::prepareUpdate()
+	 */
+	public function testPrepareUpdate_null() {
+		$page = $this->getExistingTestPage( __METHOD__ );
+		$rev1 = $page->getRevisionRecord();
+
+		$page = $this->getPage( $page->getTitle() );
+		$updater1 = $this->getDerivedPageDataUpdater( $page, $rev1 );
+
+		$editResult = new EditResult(
+			false, $rev1->getId(), null, null, null, false, true, []
+		);
+
+		$options = [
+			'editResult' => $editResult,
+			'changed' => false
+		];
+		$updater1->grabCurrentRevision();
+		$updater1->prepareContent(
+			$rev1->getUser(),
+			RevisionSlotsUpdate::newFromContent( [
+				SlotRecord::MAIN => $rev1->getMainContentRaw()
+			] )
+		);
+		$updater1->prepareUpdate( $rev1, $options );
+
+		$this->assertTrue( $updater1->isUpdatePrepared() );
+		$this->assertTrue( $updater1->isContentPrepared() );
+		$this->assertFalse( $updater1->isCreation() );
+		$this->assertFalse( $updater1->isChange() );
+
+		$this->expectDomainEvent(
+			PageRevisionUpdatedEvent::TYPE, 1,
+			static function ( PageRevisionUpdatedEvent $event ) use ( $rev1, $editResult ) {
+				Assert::assertSame( $rev1, $event->getLatestRevisionAfter() );
+				Assert::assertSame( $rev1->getId(), $event->getLatestRevisionBefore()->getId() );
+				Assert::assertSame( $editResult, $event->getEditResult() );
+				Assert::assertFalse( $event->isCreation() );
+				Assert::assertFalse( $event->changedLatestRevisionId() );
+				Assert::assertTrue( $event->isReconciliationRequest() );
+			}
+		);
+
+		$updater1->doUpdates();
+	}
+
 	/**
 	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::prepareUpdate()
 	 */
@@ -820,7 +876,7 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 		return $rev;
 	}
 
-	public function provideIsReusableFor() {
+	public static function provideIsReusableFor() {
 		$title = PageIdentityValue::localIdentity( 1234, NS_MAIN, __CLASS__ );
 
 		$user1 = new UserIdentityValue( 111, 'Alice' );
@@ -838,12 +894,12 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 		$update2 = new RevisionSlotsUpdate();
 		$update2->modifyContent( SlotRecord::MAIN, $content2 );
 
-		$rev1 = $this->makeRevision( $title, $update1, $user1, 'rev1', 11 );
-		$rev1b = $this->makeRevision( $title, $update1b, $user1, 'rev1', 11 );
+		$rev1 = [ $title, $update1, $user1, 'rev1', 11 ];
+		$rev1b = [ $title, $update1b, $user1, 'rev1', 11 ];
 
-		$rev2 = $this->makeRevision( $title, $update2, $user1, 'rev2', 12 );
-		$rev2x = $this->makeRevision( $title, $update2, $user2, 'rev2', 12 );
-		$rev2y = $this->makeRevision( $title, $update2, $user1, 'rev2', 122 );
+		$rev2 = [ $title, $update2, $user1, 'rev2', 12 ];
+		$rev2x = [ $title, $update2, $user2, 'rev2', 12 ];
+		$rev2y = [ $title, $update2, $user1, 'rev2', 122 ];
 
 		yield 'any' => [
 			'$prepUser' => null,
@@ -963,10 +1019,10 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 	 */
 	public function testIsReusableFor(
 		?UserIdentity $prepUser,
-		?RevisionRecord $prepRevision,
+		?array $prepRevision,
 		?RevisionSlotsUpdate $prepUpdate,
 		?UserIdentity $forUser,
-		?RevisionRecord $forRevision,
+		?array $forRevision,
 		?RevisionSlotsUpdate $forUpdate,
 		$forParent,
 		$isReusable
@@ -978,7 +1034,10 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 		}
 
 		if ( $prepRevision ) {
-			$updater->prepareUpdate( $prepRevision );
+			$updater->prepareUpdate( $this->makeRevision( ...$prepRevision ) );
+		}
+		if ( $forRevision ) {
+			$forRevision = $this->makeRevision( ...$forRevision );
 		}
 
 		$this->assertSame(
@@ -1068,14 +1127,19 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
+	 * @dataProvider provideDoUpdatesParams
+	 *
 	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::doUpdates()
 	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::doSecondaryDataUpdates()
 	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::doParserCacheUpdate()
 	 */
-	public function testDoUpdates() {
+	public function testDoUpdates(
+		bool $simulateNullEdit,
+		bool $simulatePageCreation
+	) {
 		$page = $this->getPage( __METHOD__ );
 
-		$content = [ SlotRecord::MAIN => new WikitextContent( 'first [[main]]' ) ];
+		$content = [ SlotRecord::MAIN => new WikitextContent( 'current [[main]]' ) ];
 
 		$content['aux'] = new WikitextContent( 'Aux [[Nix]]' );
 
@@ -1087,9 +1151,41 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 			);
 		}
 
-		$rev = $this->createRevision( $page, 'first', $content );
+		// If simulating a null edit, set up a previous revision with the same content as our change.
+		// Otherwise, initialize the previous revision with different content unless simulating page creation,
+		// in which case no previous revision should be created at all.
+		if ( !$simulatePageCreation ) {
+			$oldContent = $simulateNullEdit ? $content : [ SlotRecord::MAIN => new WikitextContent( 'first [[main]]' ) ];
+			$firstRev = $this->createRevision( $page, 'first', $oldContent );
+		} else {
+			$firstRev = null;
+		}
+
+		$slotsUpdate = RevisionSlotsUpdate::newFromContent( $content );
+
+		$updater = $this->getServiceContainer()
+			->getPageUpdaterFactory()
+			->newDerivedPageDataUpdater( $page );
+		$updater->prepareContent( $this->getTestUser()->getUserIdentity(), $slotsUpdate );
+
+		// Don't create a new revision if simulating a null edit.
+		if ( $simulateNullEdit ) {
+			$rev = $firstRev;
+		} else {
+			$rev = $this->createRevision( $page, 'current', $content );
+		}
 		$pageId = $page->getId();
 
+		$listenerCalled = 0;
+		$this->getServiceContainer()->getDomainEventSource()->registerListener(
+			PageRevisionUpdatedEvent::TYPE,
+			static function ( PageRevisionUpdatedEvent $event ) use ( &$listenerCalled, $page ) {
+				$listenerCalled++;
+
+				Assert::assertTrue( $page->isSamePageAs( $event->getPage() ) );
+			}
+		);
+
 		$oldStats = $this->getDb()->newSelectQueryBuilder()
 			->select( '*' )
 			->from( 'site_stats' )
@@ -1104,10 +1200,10 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 		$pcache = $this->getServiceContainer()->getParserCache();
 		$pcache->deleteOptionsKey( $page );
 
-		$updater = $this->getDerivedPageDataUpdater( $page, $rev );
 		$updater->setArticleCountMethod( 'link' );
 
 		$options = []; // TODO: test *all* the options...
+
 		$updater->prepareUpdate( $rev, $options );
 
 		$updater->doUpdates();
@@ -1142,9 +1238,22 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 			->from( 'site_stats' )
 			->where( '1=1' )
 			->fetchRow();
-		$this->assertSame( $oldStats->ss_total_pages + 1, (int)$stats->ss_total_pages );
-		$this->assertSame( $oldStats->ss_total_edits + 1, (int)$stats->ss_total_edits );
-		$this->assertSame( $oldStats->ss_good_articles + 1, (int)$stats->ss_good_articles );
+		if ( $simulatePageCreation ) {
+			$this->assertSame( $oldStats->ss_total_pages + 1, (int)$stats->ss_total_pages );
+			$this->assertSame( $oldStats->ss_good_articles + 1, (int)$stats->ss_good_articles );
+		} else {
+			$this->assertSame( $oldStats->ss_total_pages, $stats->ss_total_pages );
+			$this->assertSame( $oldStats->ss_good_articles, $stats->ss_good_articles );
+		}
+
+		if ( !$simulateNullEdit ) {
+			$this->assertSame( $oldStats->ss_total_edits + 1, (int)$stats->ss_total_edits );
+		} else {
+			$this->assertSame( $oldStats->ss_total_edits, $stats->ss_total_edits );
+		}
+
+		$this->runDeferredUpdates();
+		$this->assertSame( 1, $listenerCalled, 'PageRevisionUpdatedEvent listener' );
 
 		// TODO: MCR: test data updates for additional slots!
 		// TODO: test update for edit without page creation
@@ -1154,7 +1263,63 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 		// TODO: test newtalk update
 		// TODO: test search update
 		// TODO: test site stats good_articles while turning the page into (or back from) a redir.
-		// TODO: test category membership update (with setRcWatchCategoryMembership())
+	}
+
+	public static function provideDoUpdatesParams(): iterable {
+		$testCases = ArrayUtils::cartesianProduct(
+			// null or non-null edit
+			[ true, false ],
+			// page creation
+			[ true, false ]
+		);
+
+		foreach ( $testCases as $params ) {
+			[ $simulateNullEdit, $simulatePageCreation ] = $params;
+
+			if ( $simulateNullEdit && $simulatePageCreation ) {
+				// Page creations cannot be null edits, so don't simulate an impossible scenario
+				continue;
+			}
+
+			$description = sprintf(
+				'%s edit%s',
+				$simulateNullEdit ? 'null' : 'non-null',
+				$simulatePageCreation ? ', page creation, ' : ''
+			);
+
+			yield $description => $params;
+		}
+	}
+
+	/**
+	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::emitEvents()
+	 */
+	public function testDispatchPageRevisionUpdatedEvent() {
+		$page = $this->getPage( __METHOD__ );
+		$content = [ SlotRecord::MAIN => new WikitextContent( 'first [[main]]' ) ];
+		$rev = $this->createRevision( $page, 'first', $content );
+
+		$listenerCalled = 0;
+		$this->getServiceContainer()->getDomainEventSource()->registerListener(
+			PageRevisionUpdatedEvent::TYPE,
+			static function ( PageRevisionUpdatedEvent $event ) use ( &$listenerCalled, $page ) {
+				$listenerCalled++;
+
+				Assert::assertTrue( $page->isSamePageAs( $event->getPage() ) );
+			}
+		);
+
+		$updater = $this->getDerivedPageDataUpdater( $page, $rev );
+		$updater->prepareUpdate( $rev );
+
+		// Dispatch PageRevisionUpdatedEvent explicitly, then assert that doUpdates()
+		// doesn't dispatch it again.
+		$updater->emitEvents();
+
+		$updater->doUpdates();
+
+		$this->runDeferredUpdates();
+		$this->assertSame( 1, $listenerCalled, 'PageRevisionUpdatedEvent listener' );
 	}
 
 	/**
@@ -1181,6 +1346,7 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 		$this->assertNotFalse( $pcache->get( $page, $updater->getCanonicalParserOptions() ) );
 
 		$this->getDb()->endAtomic( __METHOD__ ); // run deferred updates
+		$this->runDeferredUpdates();
 
 		$this->assertSame( 0, DeferredUpdates::pendingUpdatesCount(), 'No pending updates' );
 	}
@@ -1217,24 +1383,157 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 		$this->assertFalse( $pcache->get( $page, $updater->getCanonicalParserOptions() ) );
 
 		$this->getDb()->endAtomic( __METHOD__ ); // run deferred updates
+		$this->runDeferredUpdates();
 
 		$this->assertSame( 0, DeferredUpdates::pendingUpdatesCount(), 'No pending updates' );
 		$this->assertNotFalse( $pcache->get( $page, $updater->getCanonicalParserOptions() ) );
 	}
 
+	/**
+	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::doUpdates()
+	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::maybeAddRecreateChangeTag
+	 */
+	public function testDoUpdatesTagsEditAsRecreatedWhenDeletionLogEntry() {
+		$page = $this->getPage( __METHOD__ );
+		$title = $this->getTitle( __METHOD__ );
+
+		$content = [ SlotRecord::MAIN => new WikitextContent( 'rev ID ver #1: {{REVISIONID}}' ) ];
+
+		// create a deletion log entry
+		$deleteLogEntry = new ManualLogEntry( 'delete', 'delete' );
+		$deleteLogEntry->setPerformer( $this->getTestUser()->getUser() );
+		$deleteLogEntry->setTarget( $title );
+		$logId = $deleteLogEntry->insert( $this->getDb() );
+		$deleteLogEntry->publish( $logId );
+
+		$rev = $this->createRevision( $page, 'first', $content );
+
+		$this->assertSame( [ 'mw-recreated' ], $this->getServiceContainer()->getChangeTagsStore()->getTags(
+			$this->getDb(), null, $rev->getId() ) );
+	}
+
+	/**
+	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::doUpdates()
+	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::maybeAddRecreateChangeTag
+	 */
+	public function testDoUpdatesDoesNotTagEditAsRecreatedWhenNotNewPageCreation() {
+		$page = $this->getPage( __METHOD__ );
+		$title = $this->getTitle( __METHOD__ );
+
+		$content = [ SlotRecord::MAIN => new WikitextContent( 'rev ID ver #1: {{REVISIONID}}' ) ];
+		$content2 = [ SlotRecord::MAIN => new WikitextContent( 'rev ID ver #2: {{REVISIONID}}' ) ];
+		$this->createRevision( $page, 'first', $content );
+
+		// create a deletion log entry
+		$deleteLogEntry = new ManualLogEntry( 'delete', 'delete' );
+		$deleteLogEntry->setPerformer( $this->getTestUser()->getUser() );
+		$deleteLogEntry->setTarget( $title );
+		$logId = $deleteLogEntry->insert( $this->getDb() );
+		$deleteLogEntry->publish( $logId );
+
+		$rev = $this->createRevision( $page, 'second', $content2 );
+
+		$this->assertSame( [], $this->getServiceContainer()->getChangeTagsStore()->getTags(
+			$this->getDb(), null, $rev->getId() ) );
+	}
+
+	/**
+	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::doUpdates()
+	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::maybeAddRecreateChangeTag
+	 */
+	public function testDoUpdatesDoesNotTagEditAsRecreatedWhenDeletionLogEntryAndUndelete() {
+		$page = $this->getPage( __METHOD__ );
+		$title = $this->getTitle( __METHOD__ );
+		$user = $this->getMutableTestUser()->getUser();
+		$mediaWikiServices = $this->getServiceContainer();
+		$changeTagsStore = $mediaWikiServices->getChangeTagsStore();
+
+		$content = [ SlotRecord::MAIN => new WikitextContent( 'rev ID ver #1: {{REVISIONID}}' ) ];
+
+		// create a revision on the page
+		$this->createRevision( $page, 'first', $content );
+		// make sure no tags on initial revision
+		$this->assertSame( [], $changeTagsStore->getTags(
+			$this->getDb(), null, $page->getRevisionRecord()->getId() ) );
+
+		// create a deletion log entry
+		$deleteLogEntry = new ManualLogEntry( 'delete', 'delete' );
+		$deleteLogEntry->setPerformer( $this->getTestUser()->getUser() );
+		$deleteLogEntry->setTarget( $title );
+		$logId = $deleteLogEntry->insert( $this->getDb() );
+		$deleteLogEntry->publish( $logId );
+		// undelete the page
+		$mediaWikiServices
+			->getUndeletePageFactory()
+			->newUndeletePage( $page, $user );
+		// ensure revision after undelete is not tagged with recreate
+		$this->assertSame( [], $changeTagsStore->getTags(
+			$this->getDb(), null, $page->getRevisionRecord()->getId() ) );
+	}
+
+	/**
+	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::doUpdates()
+	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::maybeAddRecreateChangeTag
+	 */
+	public function testDoUpdatesDoesNotTagEditAsRecreatedWhenNoDeletionLogEntry() {
+		$page = $this->getPage( __METHOD__ );
+
+		$content = [ SlotRecord::MAIN => new WikitextContent( 'rev ID ver #1: {{REVISIONID}}' ) ];
+		$rev = $this->createRevision( $page, 'first', $content );
+
+		$this->assertSame( [], $this->getServiceContainer()->getChangeTagsStore()->getTags(
+			$this->getDb(), null, $rev->getId() ) );
+	}
+
+	/**
+	 * See T385792
+	 *
+	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::doUpdates()
+	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::maybeAddRecreateChangeTag
+	 */
+	public function testDoUpdatesDoesNotTagEditAsRecreatedWhenDeletionLogEntryActionHidden() {
+		$page = $this->getPage( __METHOD__ );
+		$title = $this->getTitle( __METHOD__ );
+
+		$content = [ SlotRecord::MAIN => new WikitextContent( 'rev ID ver #1: {{REVISIONID}}' ) ];
+
+		// create a deletion log entry
+		$deleteLogEntry = new ManualLogEntry( 'delete', 'delete' );
+		$deleteLogEntry->setPerformer( $this->getTestUser()->getUser() );
+		$deleteLogEntry->setTarget( $title );
+
+		// hide the target of the deletion log entry
+		$deleteLogEntry->setDeleted( LogPage::DELETED_ACTION );
+
+		$logId = $deleteLogEntry->insert( $this->getDb() );
+		$deleteLogEntry->publish( $logId );
+
+		$rev = $this->createRevision( $page, 'first', $content );
+
+		$this->assertSame( [], $this->getServiceContainer()->getChangeTagsStore()->getTags(
+			$this->getDb(), null, $rev->getId() ) );
+	}
+
 	public static function provideEnqueueRevertedTagUpdateJob() {
 		return [
-			'approved' => [ true, 1 ],
-			'not approved' => [ false, 0 ]
+			'not patrolled' => [ true, 0, 0 ],
+			'patrolled' => [ true, RecentChange::PRC_AUTOPATROLLED, 1 ],
+			'autopatrolled' => [ true, RecentChange::PRC_AUTOPATROLLED, 1 ],
+			'patrolling disabled' => [ false, 0, 1 ]
 		];
 	}
 
 	/**
 	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::doUpdates
-	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::maybeEnqueueRevertedTagUpdateJob
+	 * @covers \MediaWiki\RecentChanges\ChangeTrackingEventIngress::updateRevertTagAfterPageUpdated
 	 * @dataProvider provideEnqueueRevertedTagUpdateJob
 	 */
-	public function testEnqueueRevertedTagUpdateJob( bool $approved, int $queueSize ) {
+	public function testEnqueueRevertedTagUpdateJob(
+		bool $useRcPatrol,
+		int $rcPatrolStatus,
+		int $expectQueueSize
+	) {
+		$this->overrideConfigValue( MainConfigNames::UseRCPatrol, $useRcPatrol );
 		$page = $this->getPage( __METHOD__ );
 
 		$content = [ SlotRecord::MAIN => new WikitextContent( '1' ) ];
@@ -1254,7 +1553,7 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 
 		$updater->prepareUpdate( $rev, [
 			'editResult' => $editResult,
-			'approved' => $approved
+			'rcPatrolStatus' => $rcPatrolStatus,
 		] );
 		$updater->doUpdates();
 
@@ -1268,30 +1567,30 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 			)
 		);
 
-		if ( $approved ) {
-			$this->assertNull(
-				$editResultCache->get( $rev->getId() ),
-				'EditResult should not be cached when the revert is approved'
-			);
-		} else {
+		if ( $useRcPatrol ) {
 			$this->assertEquals(
 				$editResult,
 				$editResultCache->get( $rev->getId() ),
-				'EditResult should be cached when the revert is not approved'
+				'EditResult should be cached if patrolling is enabled'
+			);
+		} else {
+			$this->assertNull(
+				$editResultCache->get( $rev->getId() ),
+				'EditResult should not be cached if patrolling is disabled'
 			);
 		}
 
 		$jobQueueGroup = $this->getServiceContainer()->getJobQueueGroup();
 		$jobQueue = $jobQueueGroup->get( 'revertedTagUpdate' );
 		$this->assertSame(
-			$queueSize,
+			$expectQueueSize,
 			$jobQueue->getSize()
 		);
 	}
 
 	/**
 	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::doParserCacheUpdate()
-	 * @covers \ParsoidCachePrewarmJob::doParsoidCacheUpdate()
+	 * @covers \MediaWiki\JobQueue\Jobs\ParsoidCachePrewarmJob::doParsoidCacheUpdate()
 	 */
 	public function testDoParserCacheUpdate() {
 		$this->overrideConfigValue(
@@ -1388,7 +1687,7 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 
 	/**
 	 * @covers \MediaWiki\Storage\DerivedPageDataUpdater::doParserCacheUpdate()
-	 * @covers \ParsoidCachePrewarmJob::doParsoidCacheUpdate()
+	 * @covers \MediaWiki\JobQueue\Jobs\ParsoidCachePrewarmJob::doParsoidCacheUpdate()
 	 */
 	public function testDoParserCacheUpdateForJavaScriptContent() {
 		$this->overrideConfigValue(
@@ -1446,7 +1745,8 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 	private function editAndUpdate( $page, $content ) {
 		$this->createRevision( $page, $content );
 		$this->getServiceContainer()->resetServiceForTesting( 'BacklinkCacheFactory' );
-		$this->runJobs();
+		DeferredUpdates::doUpdates();
+		$this->runJobs( [ 'minJobs' => 0 ] );
 	}
 
 	/**
@@ -1470,4 +1770,70 @@ class DerivedPageDataUpdaterTest extends MediaWikiIntegrationTestCase {
 		$this->assertGreaterThan( $oldTouched, $newTouched );
 	}
 
+	public static function provideNewTalk() {
+		yield 'Talk page edit' => [
+			'NewTalk TestAuthor',
+			'User_talk:NewTalk_TestUser',
+			'NewTalk TestUser',
+			true,
+		];
+		yield 'Own talk page' => [
+			'NewTalk TestUser',
+			'User_talk:NewTalk_TestUser',
+			'NewTalk TestUser',
+			false,
+		];
+		yield 'IP user page' => [
+			'NewTalk TestAuthor',
+			'User_talk:192.168.0.1',
+			'192.168.0.1',
+			true,
+		];
+		yield 'User talk subpage' => [
+			'NewTalk TestAuthor',
+			'User_talk:NewTalk_TestUser/sandbox',
+			'NewTalk TestUser',
+			false,
+		];
+		yield 'Not talk page' => [
+			'NewTalk TestAuthor',
+			'User:NewTalk_TestUser',
+			'NewTalk TestUser',
+			false,
+		];
+	}
+
+	private function createUser( string $name ) {
+		$userFactory = $this->getServiceContainer()->getUserFactory();
+
+		$user = $userFactory->newFromName( $name );
+		if ( !$user ) {
+			$user = $userFactory->newAnonymous( $name );
+		} elseif ( !$user->getId() ) {
+			$user->addToDatabase();
+		}
+
+		return $user;
+	}
+
+	/**
+	 * @dataProvider provideNewTalk
+	 */
+	public function testNewTalk( string $authorName, string $pageName, string $recipientName, bool $expected ) {
+		$author = $this->createUser( $authorName );
+		$recipient = $this->createUser( $recipientName );
+
+		$notificationManager = $this->getServiceContainer()->getTalkPageNotificationManager();
+		$notificationManager->clearForPageView( $recipient );
+
+		$page = $this->getPage( $pageName );
+
+		$content = new WikitextContent( 'Hi there!' );
+		$this->createRevision( $page, 'Testing', $content, $author );
+		DeferredUpdates::doUpdates();
+
+		$hasNewMessage = $notificationManager->userHasNewMessages( $recipient );
+		$this->assertSame( $expected, $hasNewMessage );
+	}
+
 }
