@@ -3,10 +3,10 @@
 namespace MediaWiki\Tests\Block;
 
 use MediaWiki\Block\BlockRestrictionStore;
-use MediaWiki\Block\DatabaseBlock;
 use MediaWiki\Block\Restriction\NamespaceRestriction;
 use MediaWiki\Block\Restriction\PageRestriction;
 use MediaWiki\Block\Restriction\Restriction;
+use MediaWiki\MainConfigNames;
 
 /**
  * @group Database
@@ -15,8 +15,7 @@ use MediaWiki\Block\Restriction\Restriction;
  */
 class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 
-	/** @var BlockRestrictionStore */
-	protected $blockRestrictionStore;
+	protected BlockRestrictionStore $blockRestrictionStore;
 
 	protected function setUp(): void {
 		parent::setUp();
@@ -24,20 +23,13 @@ class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 		$this->blockRestrictionStore = $this->getServiceContainer()->getBlockRestrictionStore();
 	}
 
-	protected function tearDown(): void {
-		$this->resetTables();
-		parent::tearDown();
-	}
-
 	/**
 	 * @covers ::loadByBlockId
 	 * @covers ::resultToRestrictions
 	 * @covers ::rowToRestriction
 	 */
 	public function testLoadMultipleRestrictions() {
-		$this->setMwGlobals( [
-			'wgBlockDisablesLogin' => false,
-		] );
+		$this->overrideConfigValue( MainConfigNames::BlockDisablesLogin, false );
 		$block = $this->insertBlock();
 
 		$pageFoo = $this->getExistingTestPage( 'Foo' );
@@ -115,11 +107,11 @@ class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 
 		$restrictions = $this->blockRestrictionStore->loadByBlockId( $block->getId() );
 
-		list( $pageRestriction ) = $restrictions;
+		[ $pageRestriction ] = $restrictions;
 		$this->assertInstanceOf( PageRestriction::class, $pageRestriction );
 		$this->assertEquals( $block->getId(), $pageRestriction->getBlockId() );
 		$this->assertEquals( $page->getId(), $pageRestriction->getValue() );
-		$this->assertEquals( $pageRestriction->getType(), PageRestriction::TYPE );
+		$this->assertEquals( PageRestriction::TYPE, $pageRestriction->getType() );
 		$this->assertEquals( $pageRestriction->getTitle()->getText(), $title );
 	}
 
@@ -137,11 +129,11 @@ class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 
 		$restrictions = $this->blockRestrictionStore->loadByBlockId( $block->getId() );
 
-		list( $namespaceRestriction ) = $restrictions;
+		[ $namespaceRestriction ] = $restrictions;
 		$this->assertInstanceOf( NamespaceRestriction::class, $namespaceRestriction );
 		$this->assertEquals( $block->getId(), $namespaceRestriction->getBlockId() );
 		$this->assertSame( NS_USER, $namespaceRestriction->getValue() );
-		$this->assertEquals( $namespaceRestriction->getType(), NamespaceRestriction::TYPE );
+		$this->assertEquals( NamespaceRestriction::TYPE, $namespaceRestriction->getType() );
 	}
 
 	/**
@@ -154,7 +146,6 @@ class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 		$pageBar = $this->getExistingTestPage( 'Bar' );
 
 		$restrictions = [
-			(object)[],
 			new PageRestriction( $block->getId(), $pageFoo->getId() ),
 			new PageRestriction( $block->getId(), $pageBar->getId() ),
 			new NamespaceRestriction( $block->getId(), NS_USER )
@@ -163,13 +154,6 @@ class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 		$result = $this->blockRestrictionStore->insert( $restrictions );
 		$this->assertTrue( $result );
 
-		$restrictions = [
-			(object)[],
-		];
-
-		$result = $this->blockRestrictionStore->insert( $restrictions );
-		$this->assertFalse( $result );
-
 		$result = $this->blockRestrictionStore->insert( [] );
 		$this->assertFalse( $result );
 	}
@@ -192,7 +176,6 @@ class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 			] );
 
 		$restrictions = [
-			(object)[],
 			new PageRestriction( $block->getId(), $pageFoo->getId() ),
 			new PageRestriction( $block->getId(), $pageBar->getId() ),
 			new NamespaceRestriction( $block->getId(), NS_USER ),
@@ -220,17 +203,15 @@ class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 		] );
 
 		$this->blockRestrictionStore->update( [
-			(object)[],
 			new PageRestriction( $block->getId(), $pageBar->getId() ),
 			new NamespaceRestriction( $block->getId(), NS_USER ),
 		] );
 
-		$db = wfGetDB( DB_REPLICA );
-		$result = $db->select(
-			[ 'ipblocks_restrictions' ],
-			[ '*' ],
-			[ 'ir_ipb_id' => $block->getId() ]
-		);
+		$result = $this->getDb()->newSelectQueryBuilder()
+			->select( [ '*' ] )
+			->from( 'ipblocks_restrictions' )
+			->where( [ 'ir_ipb_id' => $block->getId() ] )
+			->fetchResultSet();
 
 		$this->assertEquals( 2, $result->numRows() );
 		$row = $result->fetchObject();
@@ -251,12 +232,11 @@ class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 			new PageRestriction( $block->getId(), $page->getId() ),
 		] );
 
-		$db = wfGetDB( DB_REPLICA );
-		$result = $db->select(
-			[ 'ipblocks_restrictions' ],
-			[ '*' ],
-			[ 'ir_ipb_id' => $block->getId() ]
-		);
+		$result = $this->getDb()->newSelectQueryBuilder()
+			->select( [ '*' ] )
+			->from( 'ipblocks_restrictions' )
+			->where( [ 'ir_ipb_id' => $block->getId() ] )
+			->fetchResultSet();
 
 		$this->assertSame( 1, $result->numRows() );
 		$row = $result->fetchObject();
@@ -274,12 +254,11 @@ class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 
 		$this->blockRestrictionStore->update( [] );
 
-		$db = wfGetDB( DB_REPLICA );
-		$result = $db->select(
-			[ 'ipblocks_restrictions' ],
-			[ '*' ],
-			[ 'ir_ipb_id' => $block->getId() ]
-		);
+		$result = $this->getDb()->newSelectQueryBuilder()
+			->select( [ '*' ] )
+			->from( 'ipblocks_restrictions' )
+			->where( [ 'ir_ipb_id' => $block->getId() ] )
+			->fetchResultSet();
 
 		$this->assertSame( 0, $result->numRows() );
 	}
@@ -300,12 +279,11 @@ class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 			new PageRestriction( $block->getId(), $page->getId() ),
 		] );
 
-		$db = wfGetDB( DB_REPLICA );
-		$result = $db->select(
-			[ 'ipblocks_restrictions' ],
-			[ '*' ],
-			[ 'ir_ipb_id' => $block->getId() ]
-		);
+		$result = $this->getDb()->newSelectQueryBuilder()
+			->select( [ '*' ] )
+			->from( 'ipblocks_restrictions' )
+			->where( [ 'ir_ipb_id' => $block->getId() ] )
+			->fetchResultSet();
 
 		$this->assertSame( 1, $result->numRows() );
 		$row = $result->fetchObject();
@@ -324,7 +302,8 @@ class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 		$this->blockRestrictionStore->insert( [
 			new PageRestriction( $block->getId(), $pageFoo->getId() ),
 		] );
-		$autoblockId = $block->doAutoblock( '127.0.0.1' );
+		$autoblockId = $this->getServiceContainer()->getDatabaseBlockStore()
+			->doAutoblock( $block, '127.0.0.1' );
 
 		// Ensure that the restrictions on the block have not changed.
 		$restrictions = $this->blockRestrictionStore->loadByBlockId( $block->getId() );
@@ -362,7 +341,8 @@ class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 		$this->blockRestrictionStore->insert( [
 			new PageRestriction( $block->getId(), $page->getId() ),
 		] );
-		$autoblockId = $block->doAutoblock( '127.0.0.1' );
+		$autoblockId = $this->getServiceContainer()->getDatabaseBlockStore()
+			->doAutoblock( $block, '127.0.0.1' );
 
 		// Ensure that the restrictions on the block have not changed.
 		$restrictions = $this->blockRestrictionStore->loadByBlockId( $block->getId() );
@@ -420,9 +400,7 @@ class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 		$restrictions = $this->blockRestrictionStore->loadByBlockId( $block->getId() );
 		$this->assertCount( 1, $restrictions );
 
-		$result = $this->blockRestrictionStore->delete(
-			array_merge( $restrictions, [ (object)[] ] )
-		);
+		$result = $this->blockRestrictionStore->delete( $restrictions );
 		$this->assertTrue( $result );
 
 		$restrictions = $this->blockRestrictionStore->loadByBlockId( $block->getId() );
@@ -449,40 +427,6 @@ class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 		$this->assertSame( [], $restrictions );
 	}
 
-	/**
-	 * @covers ::deleteByParentBlockId
-	 */
-	public function testDeleteByParentBlockId() {
-		// Create a block with no autoblock.
-		$block = $this->insertBlock();
-		$page = $this->getExistingTestPage( 'Foo' );
-		$this->blockRestrictionStore->insert( [
-			new PageRestriction( $block->getId(), $page->getId() ),
-		] );
-		$autoblockId = $block->doAutoblock( '127.0.0.1' );
-
-		// Ensure that the restrictions on the block have not changed.
-		$restrictions = $this->blockRestrictionStore->loadByBlockId( $block->getId() );
-		$this->assertCount( 1, $restrictions );
-
-		// Ensure that the restrictions on the autoblock are the same as the block.
-		$restrictions = $this->blockRestrictionStore->loadByBlockId( $autoblockId );
-		$this->assertCount( 1, $restrictions );
-
-		// Remove all of the restrictions on the autoblock (but leave the block unchanged).
-		$result = $this->blockRestrictionStore->deleteByParentBlockId( $block->getId() );
-		// NOTE: commented out until https://gerrit.wikimedia.org/r/c/mediawiki/core/+/469324 is merged
-		//$this->assertTrue( $result );
-
-		// Ensure that the restrictions on the block have not changed.
-		$restrictions = $this->blockRestrictionStore->loadByBlockId( $block->getId() );
-		$this->assertCount( 1, $restrictions );
-
-		// Ensure that the restrictions on the autoblock have been removed.
-		$restrictions = $this->blockRestrictionStore->loadByBlockId( $autoblockId );
-		$this->assertSame( [], $restrictions );
-	}
-
 	/**
 	 * @covers ::equals
 	 * @dataProvider equalsDataProvider
@@ -495,15 +439,13 @@ class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 		$this->assertSame( $expected, $this->blockRestrictionStore->equals( $a, $b ) );
 	}
 
-	public function equalsDataProvider() {
+	public static function equalsDataProvider() {
 		return [
 			[
 				[
-					(object)[],
 					new PageRestriction( 1, 1 ),
 				],
 				[
-					(object)[],
 					new PageRestriction( 1, 2 )
 				],
 				false,
@@ -562,15 +504,14 @@ class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 	 */
 	public function testSetBlockId() {
 		$restrictions = [
-			(object)[],
 			new PageRestriction( 1, 1 ),
 			new PageRestriction( 1, 2 ),
 			new NamespaceRestriction( 1, NS_USER ),
 		];
 
+		$this->assertSame( 1, $restrictions[0]->getBlockId() );
 		$this->assertSame( 1, $restrictions[1]->getBlockId() );
 		$this->assertSame( 1, $restrictions[2]->getBlockId() );
-		$this->assertSame( 1, $restrictions[3]->getBlockId() );
 
 		$result = $this->blockRestrictionStore->setBlockId( 2, $restrictions );
 
@@ -583,30 +524,25 @@ class BlockRestrictionStoreTest extends \MediaWikiLangTestCase {
 		$badActor = $this->getTestUser()->getUser();
 		$sysop = $this->getTestSysop()->getUser();
 
-		$block = new DatabaseBlock( [
-			'address' => $badActor->getName(),
-			'user' => $badActor->getId(),
-			'by' => $sysop,
-			'expiry' => 'infinity',
-			'sitewide' => 0,
-			'enableAutoblock' => true,
-		] );
-
-		$this->getServiceContainer()->getDatabaseBlockStore()->insertBlock( $block );
-
-		return $block;
+		return $this->getServiceContainer()->getDatabaseBlockStore()
+			->insertBlockWithParams( [
+				'address' => $badActor,
+				'by' => $sysop,
+				'expiry' => 'infinity',
+				'sitewide' => 0,
+				'enableAutoblock' => true,
+			] );
 	}
 
 	protected function insertRestriction( $blockId, $type, $value ) {
-		$this->db->insert( 'ipblocks_restrictions', [
-			'ir_ipb_id' => $blockId,
-			'ir_type' => $type,
-			'ir_value' => $value,
-		] );
-	}
-
-	protected function resetTables() {
-		$this->db->delete( 'ipblocks', '*', __METHOD__ );
-		$this->db->delete( 'ipblocks_restrictions', '*', __METHOD__ );
+		$this->getDb()->newInsertQueryBuilder()
+			->insertInto( 'ipblocks_restrictions' )
+			->row( [
+				'ir_ipb_id' => $blockId,
+				'ir_type' => $type,
+				'ir_value' => $value,
+			] )
+			->caller( __METHOD__ )
+			->execute();
 	}
 }
