@@ -1,13 +1,31 @@
 <?php
 
+use MediaWiki\CommentStore\CommentStoreComment;
+use MediaWiki\Content\Content;
+use MediaWiki\Content\ContentHandler;
+use MediaWiki\Content\WikitextContent;
+use MediaWiki\Context\DerivativeContext;
+use MediaWiki\Context\RequestContext;
+use MediaWiki\MainConfigNames;
+use MediaWiki\Output\OutputPage;
+use MediaWiki\Page\Article;
+use MediaWiki\Page\WikiPage;
+use MediaWiki\Parser\ParserOptions;
+use MediaWiki\Parser\ParserOutput;
+use MediaWiki\Request\FauxRequest;
 use MediaWiki\Revision\MutableRevisionRecord;
 use MediaWiki\Revision\RevisionRecord;
 use MediaWiki\Revision\SlotRecord;
+use MediaWiki\Status\Status;
+use MediaWiki\Title\Title;
+use MediaWiki\User\User;
+use MediaWiki\Utils\MWTimestamp;
 use PHPUnit\Framework\MockObject\MockObject;
 use Wikimedia\TestingAccessWrapper;
 
 /**
- * @covers \Article::view()
+ * @covers \MediaWiki\Page\Article::view()
+ * @group Database
  */
 class ArticleViewTest extends MediaWikiIntegrationTestCase {
 
@@ -28,7 +46,6 @@ class ArticleViewTest extends MediaWikiIntegrationTestCase {
 	 * @param RevisionRecord[] &$revisions will be filled with the RevisionRecord for $content.
 	 *
 	 * @return WikiPage
-	 * @throws MWException
 	 */
 	private function getPage( $title, array $revisionContents = [], array &$revisions = [] ) {
 		if ( is_string( $title ) ) {
@@ -65,8 +82,8 @@ class ArticleViewTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
-	 * @covers Article::getOldId()
-	 * @covers Article::getRevIdFetched()
+	 * @covers \MediaWiki\Page\Article::getOldId()
+	 * @covers \MediaWiki\Page\Article::getRevIdFetched()
 	 */
 	public function testGetOldId() {
 		$revisions = [];
@@ -139,9 +156,9 @@ class ArticleViewTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
-	 * @covers Article::getPage
-	 * @covers WikiPage::getRedirectTarget
-	 * @covers \Mediawiki\Page\RedirectLookup::getRedirectTarget
+	 * @covers \MediaWiki\Page\Article::getPage
+	 * @covers \MediaWiki\Page\WikiPage::getRedirectTarget
+	 * @covers \MediaWiki\Page\RedirectLookup::getRedirectTarget
 	 */
 	public function testViewRedirect() {
 		$target = Title::makeTitle( $this->getDefaultWikitextNS(), 'Test_Target' );
@@ -154,13 +171,14 @@ class ArticleViewTest extends MediaWikiIntegrationTestCase {
 		$article->view();
 
 		$redirectStore = $this->getServiceContainer()->getRedirectStore();
+		$titleFormatter = $this->getServiceContainer()->getTitleFormatter();
 
 		$this->assertNotNull(
-			$redirectStore->getRedirectTarget( $article->getPage() )->getPrefixedDBkey()
+			$redirectStore->getRedirectTarget( $article->getPage() )
 		);
 		$this->assertSame(
 			$target->getPrefixedDBkey(),
-			$redirectStore->getRedirectTarget( $article->getPage() )->getPrefixedDBkey()
+			$titleFormatter->getPrefixedDBkey( $redirectStore->getRedirectTarget( $article->getPage() ) )
 		);
 
 		$output = $article->getContext()->getOutput();
@@ -207,8 +225,7 @@ class ArticleViewTest extends MediaWikiIntegrationTestCase {
 		$content->method( 'getModel' )
 			->willReturn( 'NotText' );
 		$content->expects( $this->never() )->method( 'getNativeData' );
-		$content->method( 'copy' )
-			->willReturn( $content );
+		$content->method( 'copy' )->willReturnSelf();
 
 		$rev = new MutableRevisionRecord( $title );
 		$rev->setId( $dummyRev->getId() );
@@ -221,7 +238,7 @@ class ArticleViewTest extends MediaWikiIntegrationTestCase {
 
 		/** @var MockObject|WikiPage $page */
 		$page = $this->getMockBuilder( WikiPage::class )
-			->onlyMethods( [ 'getRevisionRecord', 'getLatest' ] )
+			->onlyMethods( [ 'getRevisionRecord', 'getLatest', 'getContentHandler' ] )
 			->setConstructorArgs( [ $title ] )
 			->getMock();
 
@@ -229,6 +246,8 @@ class ArticleViewTest extends MediaWikiIntegrationTestCase {
 			->willReturn( $rev );
 		$page->method( 'getLatest' )
 			->willReturn( $rev->getId() );
+		$page->method( 'getContentHandler' )
+			->willReturn( $mockHandler );
 
 		$article = Article::newFromWikiPage( $page, RequestContext::getMain() );
 		$article->getContext()->getOutput()->setTitle( $page->getTitle() );
@@ -260,21 +279,16 @@ class ArticleViewTest extends MediaWikiIntegrationTestCase {
 	}
 
 	public function testViewOfOldRevisionFromCache() {
-		$this->setMwGlobals( [
-			'wgOldRevisionParserCacheExpireTime' => 100500,
-			'wgMainWANCache' => 'main',
-			'wgWANObjectCaches' => [
-				'main' => [
-					'class' => WANObjectCache::class,
-					'cacheId' => 'hash',
-				],
-			],
+		$this->overrideConfigValues( [
+			MainConfigNames::OldRevisionParserCacheExpireTime => 100500,
+			MainConfigNames::MainCacheType => CACHE_HASH,
 		] );
 
 		$revisions = [];
 		$page = $this->getPage( __METHOD__, [ 1 => 'Test A', 2 => 'Test B' ], $revisions );
 		$idA = $revisions[1]->getId();
-		$this->setMwGlobals( 'wgTitle', $page->getTitle() );
+		$context = RequestContext::getMain();
+		$context->setTitle( $page->getTitle() );
 
 		// View the revision once (to get it into the cache)
 		$article = new Article( $page->getTitle(), $idA );
@@ -282,7 +296,6 @@ class ArticleViewTest extends MediaWikiIntegrationTestCase {
 
 		// Reset the output page and view the revision again (from ParserCache)
 		$article = new Article( $page->getTitle(), $idA );
-		$context = RequestContext::getMain();
 		$context->setOutput( new OutputPage( $context ) );
 		$article->setContext( $context );
 
@@ -296,7 +309,7 @@ class ArticleViewTest extends MediaWikiIntegrationTestCase {
 		$article->view();
 		$output = $article->getContext()->getOutput();
 		$this->assertStringContainsString( 'Test A', $this->getHtml( $output ) );
-		$this->assertSame( 1, substr_count( $output->getSubtitle(), 'class="mw-revision warningbox"' ) );
+		$this->assertSame( 1, substr_count( $output->getSubtitle(), 'cdx-message--warning' ) );
 		$this->assertSame( $idA, $output->getRevisionId() );
 		$this->assertSame( $idA, $outputPageBeforeHTMLRevisionId );
 		$this->assertSame( $revisions[1]->getTimestamp(), $output->getRevisionTimestamp() );
@@ -317,6 +330,39 @@ class ArticleViewTest extends MediaWikiIntegrationTestCase {
 		$this->assertStringContainsString( 'id="mw-revision-nav"', $output->getSubtitle() );
 	}
 
+	public function testViewOfCurrentRevisionDirty() {
+		$this->overrideConfigValue(
+			MainConfigNames::PoolCounterConf,
+			[
+				'ArticleView' => [
+					'class' => MockPoolCounterFailing::class,
+				]
+			]
+		);
+
+		$revisions = [];
+		$page = $this->getPage( __METHOD__, [ 1 => 'Test A' ], $revisions );
+		$idA = $revisions[1]->getId();
+
+		// Do the next edit without ParserCache produce an outdated cache entry
+		$parserCacheFactory = $this->getServiceContainer()->getParserCacheFactory();
+		$this->overrideConfigValue( MainConfigNames::ParserCacheType, CACHE_NONE );
+		$latestEditStatus = $this->editPage( $page, 'Test B' );
+		// Restore the old cache instance with the now outdated cache entry
+		$this->setService( 'ParserCacheFactory', $parserCacheFactory );
+
+		// Request the article for the latest
+		$article = new Article( $page->getTitle() );
+		$article->getContext()->getOutput()->setTitle( $page->getTitle() );
+		$article->view();
+
+		// Expected the old values to return
+		$output = $article->getContext()->getOutput();
+		$this->assertStringContainsString( 'Test A', $this->getHtml( $output ) );
+		$this->assertSame( $idA, $output->getRevisionId() );
+		$this->assertSame( $revisions[1]->getTimestamp(), $output->getRevisionTimestamp() );
+	}
+
 	public function testViewOfMissingRevision() {
 		$revisions = [];
 		$page = $this->getPage( __METHOD__, [ 1 => 'Test A' ], $revisions );
@@ -361,23 +407,107 @@ class ArticleViewTest extends MediaWikiIntegrationTestCase {
 
 		$revDelList = $this->getRevDelRevisionList( $page->getTitle(), $idA );
 		$revDelList->setVisibility( [
-			'value' => [ RevisionRecord::DELETED_TEXT => 1 ],
+			'value' => [
+				RevisionRecord::DELETED_TEXT => 1,
+				RevisionRecord::DELETED_COMMENT => 1,
+				RevisionRecord::DELETED_USER => 1,
+			],
 			'comment' => "Testing",
 		] );
 
+		$realContext = RequestContext::getMain();
+		$oldUser = $realContext->getUser();
+		$oldLanguage = $realContext->getLanguage();
+
 		$article = new Article( $page->getTitle(), $idA );
-		$context = new DerivativeContext( $article->getContext() );
+		$context = new DerivativeContext( $realContext );
 		$article->setContext( $context );
 		$context->getOutput()->setTitle( $page->getTitle() );
 		$context->getRequest()->setVal( 'unhide', 1 );
 		$context->setUser( $this->getTestUser( [ 'sysop' ] )->getUser() );
+
+		// Need global user set to sysop, global state in Linker::revUserTools/Linker::revComment (T309479)
+		$realContext->setUser( $context->getUser() );
+		// Language is resetted in setUser
+		$this->setUserLang( $oldLanguage );
+
 		$article->view();
 
 		$output = $article->getContext()->getOutput();
-		$this->assertStringContainsString( 'rev-deleted-text-view', $this->getHtml( $output ) );
+		$subtitle = $output->getSubtitle();
+		$html = $this->getHtml( $output );
 
-		$this->assertStringContainsString( 'Test A', $this->getHtml( $output ) );
-		$this->assertStringNotContainsString( 'Test B', $this->getHtml( $output ) );
+		// Test that oldid is select, not the current version
+		$this->assertStringNotContainsString( 'Test B', $html );
+
+		// Warning about rev-del must exists
+		$this->assertStringContainsString( 'rev-deleted-text-view', $html );
+
+		// Test for the hidden values
+		$this->assertStringContainsString( 'Test A', $html );
+		$this->assertStringContainsString( $revisions[1]->getUser()->getName(), $subtitle );
+		$this->assertStringContainsString( '(parentheses: Rev 1)', $subtitle );
+
+		// Should not contain the rev-del messages
+		$this->assertStringNotContainsString( '(rev-deleted-user)', $subtitle );
+		$this->assertStringNotContainsString( '(rev-deleted-comment)', $subtitle );
+
+		$realContext->setUser( $oldUser );
+	}
+
+	public function testHiddenViewOfDeletedRevision() {
+		$revisions = [];
+		$page = $this->getPage( __METHOD__, [ 1 => 'Test A', 2 => 'Test B' ], $revisions );
+		$idA = $revisions[1]->getId();
+
+		$revDelList = $this->getRevDelRevisionList( $page->getTitle(), $idA );
+		$revDelList->setVisibility( [
+			'value' => [
+				RevisionRecord::DELETED_TEXT => 1,
+				RevisionRecord::DELETED_COMMENT => 1,
+				RevisionRecord::DELETED_USER => 1,
+			],
+			'comment' => "Testing",
+		] );
+
+		$realContext = RequestContext::getMain();
+		$oldUser = $realContext->getUser();
+		$oldLanguage = $realContext->getLanguage();
+
+		$article = new Article( $page->getTitle(), $idA );
+		$context = new DerivativeContext( $realContext );
+		$article->setContext( $context );
+		$context->getOutput()->setTitle( $page->getTitle() );
+		// No unhide=1 is set in this test case
+		$context->setUser( $this->getTestUser( [ 'sysop' ] )->getUser() );
+
+		// Need global user set to sysop, global state in Linker::revUserTools/Linker::revComment (T309479)
+		$realContext->setUser( $context->getUser() );
+		// Language is resetted in setUser
+		$this->setUserLang( $oldLanguage );
+
+		$article->view();
+
+		$output = $article->getContext()->getOutput();
+		$subtitle = $output->getSubtitle();
+		$html = $this->getHtml( $output );
+
+		// Test that oldid is select, not the current version
+		$this->assertStringNotContainsString( 'Test B', $html );
+
+		// Warning about rev-del must exists
+		$this->assertStringContainsString( 'rev-deleted-text-unhide', $html );
+
+		// Test for the rev-del messages
+		$this->assertStringContainsString( '(rev-deleted-user)', $subtitle );
+		$this->assertStringContainsString( '(rev-deleted-comment)', $subtitle );
+
+		// Should not contain the hidden values
+		$this->assertStringNotContainsString( 'Test A', $html );
+		$this->assertStringNotContainsString( $revisions[1]->getUser()->getName(), $subtitle );
+		$this->assertStringNotContainsString( '(parentheses: Rev 1)', $subtitle );
+
+		$realContext->setUser( $oldUser );
 	}
 
 	public function testViewMissingPage() {
@@ -477,7 +607,9 @@ class ArticleViewTest extends MediaWikiIntegrationTestCase {
 				$outputDone = new ParserOutput( 'Hook Text' );
 				$outputDone->setTitleText( 'Hook Title' );
 
-				$articlePage->getContext()->getOutput()->addParserOutput( $outputDone );
+				$context = $articlePage->getContext();
+				$parserOptions = ParserOptions::newFromContext( $context );
+				$context->getOutput()->addParserOutput( $outputDone, $parserOptions );
 			}
 		);
 
@@ -545,7 +677,7 @@ class ArticleViewTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
-	 * @covers \Article::showViewError()
+	 * @covers \MediaWiki\Page\Article::showViewError()
 	 */
 	public function testViewLatestError() {
 		$page = $this->getPage( __METHOD__, [ 1 => 'Test A' ] );
@@ -577,7 +709,7 @@ class ArticleViewTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
-	 * @covers \Article::showViewError()
+	 * @covers \MediaWiki\Page\Article::showViewError()
 	 */
 	public function testViewOldError() {
 		$revisions = [];
@@ -604,15 +736,60 @@ class ArticleViewTest extends MediaWikiIntegrationTestCase {
 
 	private function getRevDelRevisionList( $title, $revisionId ) {
 		$services = $this->getServiceContainer();
+		$context = new DerivativeContext( RequestContext::getMain() );
+		$context->setUser(
+			$this->getTestUser( [ 'sysop' ] )->getUser()
+		);
 		return new RevDelRevisionList(
-			RequestContext::getMain(),
+			$context,
 			$title,
 			[ $revisionId ],
-			$services->getDBLoadBalancerFactory(),
+			$services->getConnectionProvider(),
 			$services->getHookContainer(),
 			$services->getHtmlCacheUpdater(),
-			$services->getRevisionStore(),
-			$services->getMainWANObjectCache()
+			$services->getRevisionStore()
+		);
+	}
+
+	/**
+	 * Test the "useParsoid" parser option and the ArticleParserOptions
+	 * hook.
+	 */
+	public function testUseParsoid() {
+		// Create an appropriate test page.
+		$title = Title::makeTitle( NS_MAIN, 'UseParsoidTest' );
+		$article = new Article( $title );
+		$page = $this->getExistingTestPage( $title );
+		$page->doUserEditContent(
+			ContentHandler::makeContent(
+				'[[Foo]]',
+				$title,
+				// Force this page to be wikitext
+				CONTENT_MODEL_WIKITEXT
+			),
+			$this->getTestSysop()->getUser(),
+			'TestUseParsoid Summary',
+			EDIT_SUPPRESS_RC
+		);
+		$article->view();
+		$html = $this->getHtml( $article->getContext()->getOutput() );
+		// Confirm that this is NOT parsoid-generated HTML
+		$this->assertStringNotContainsString(
+			'rel="mw:WikiLink"',
+			$html
+		);
+
+		// Now enable Parsoid via the ArticleParserOptions hook
+		$article = new Article( $title );
+		$this->setTemporaryHook( 'ArticleParserOptions', static function ( $article, $popts ) {
+			$popts->setUseParsoid();
+		} );
+		$article->view();
+		$html = $this->getHtml( $article->getContext()->getOutput() );
+		// Look for a marker that this is Parsoid-generated HTML
+		$this->assertStringContainsString(
+			'rel="mw:WikiLink"',
+			$html
 		);
 	}
 }
