@@ -1,5 +1,6 @@
 <?php
 
+use MediaWiki\Content\WikitextContent;
 use MediaWiki\Context\RequestContext;
 use MediaWiki\MainConfigNames;
 use MediaWiki\Specials\SpecialMyLanguage;
@@ -31,7 +32,9 @@ class SpecialMyLanguageTest extends MediaWikiIntegrationTestCase {
 		$pageContent = [
 			'Page/Redirect' => '#REDIRECT [[Page/Another#Section]]',
 		];
-		$user = $this->getTestSysop()->getAuthority();
+		$user = $this->getTestSysop()->getUser();
+		$context = RequestContext::getMain();
+		$context->setUser( $user );
 		foreach ( $titles as $title ) {
 			$this->editPage(
 				$title,
@@ -42,7 +45,7 @@ class SpecialMyLanguageTest extends MediaWikiIntegrationTestCase {
 			);
 			if ( isset( $pageLang[$title] ) ) {
 				SpecialPageLanguage::changePageLanguage(
-					RequestContext::getMain(), Title::newFromText( $title ), $pageLang[$title], 'Test' );
+					$context, Title::newFromText( $title ), $pageLang[$title], 'Test' );
 			}
 		}
 	}
